@@ -1,5 +1,18 @@
-/* $Id: Draw.c,v 1.18 1996/10/29 00:31:23 venkat Exp $ */
-/* $Log: Draw.c,v $
+/* $Id: Draw.c,v 1.21 2000/06/12 04:05:00 mhucka Exp $ */
+/*
+ * $Log: Draw.c,v $
+ * Revision 1.21  2000/06/12 04:05:00  mhucka
+ * 1) Removed nested comments in RCS/CVS log lines, to quiet down the
+ *    IRIX cc compiler.
+ * 2) XoDrawLeave used variable "pix" before setting value.
+ * 3) Fixed obvious case of using "==" when "=" was meant.
+ *
+ * Revision 1.20  2000/05/02 06:06:41  mhucka
+ * Added type casts for certain function call arguments.
+ *
+ * Revision 1.19  1998/07/15 06:17:26  dhb
+ * Upi update
+ *
  * Revision 1.18  1996/10/29 00:31:23  venkat
  * Removed-parameter-type-from-declaration-of-atof-for-linux,-which-was-left-there-unintentionally-in-the-previous-check-in
  *
@@ -111,6 +124,9 @@
 double atof();
 #endif
 #endif /* Linux || __FreeBSD__ */
+#if defined(__GNUC__) && defined(decalpha)
+#include <stdlib.h>
+#endif /* decalpha */
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 #include "Xo/XoDefs.h"
@@ -431,7 +447,7 @@ static Boolean SetValues(curr,req,new,args,num_args)
 	/* Setting aw is illegal in this version */
 	if (dnw->draw.matrix!=dcw->draw.matrix) {
 		if (dnw->draw.matrix == NULL) {
-			dnw->draw.matrix == dcw->draw.matrix;
+			dnw->draw.matrix = dcw->draw.matrix;
 			return(False);
 		}
 		return(True);
@@ -458,7 +474,7 @@ static void Resize(w)
   int num_children = dw->composite.num_children;
   PixObjectClass class;
 
-	if (!XtIsRealized(dw))
+	if (!XtIsRealized((Widget) dw))
 		return;
 
 	/** This call has to preceed the actual projection of the
@@ -500,7 +516,7 @@ static void Redisplay (w, e, region)
   dw->coredraw.drawflags &= ~XO_DRAW_FORCE_EXPOSE;
 
 	if ((dw->coredraw.drawflags & XO_DRAW_REFRESHABLE_NOT) ||
-		!XtIsRealized(dw))
+		!XtIsRealized((Widget) dw))
 		return;
 
 	/* handle children */
@@ -603,22 +619,28 @@ static void XoDrawLeave(w,event)
     Widget  w;
     XButtonEvent    *event;
 {
-  DrawWidget dw = (DrawWidget) w;
-  Widget pix;
-  static XoEventInfo    info;
+    DrawWidget dw = (DrawWidget) w;
+    Widget pix;
+    static XoEventInfo info;
+
+    /* call the callbacks for a possible drag-drop operation
+    ** within the draw widget */
+    info.event = XoLeave;
+    info.ret = "";
+    info.x= event->x;
+    info.y = event->y;
+    info.z = 0;
  
-        /* call the callbacks for a possible drag-drop operation
-        ** within the draw widget */
-        info.event = XoLeave;
-        info.ret = "";
-        info.x= event->x;
-        info.y = event->y;
-        info.z = 0;
- 
-        /* calling the drag callback */
-        if (pix == w)
-            XtCallCallbackList(w,dw->coredraw.callbacks,
-                (XtPointer)&info);
+    /* 2000-06-10 <mhucka@bbb.caltech.edu> I don't know whether the
+    ** following line is correct, but previously, pix was never assigned,
+    ** so something needed to be done here.  XoDrawEnter() uses this,
+    ** so let's try it here too.
+    */
+    pix = XoFindNearestPix(w,event);
+    /* calling the drag callback */
+    if (pix == w)
+	XtCallCallbackList(w,dw->coredraw.callbacks,
+			   (XtPointer)&info);
 }
  
 
