@@ -1,5 +1,15 @@
-/* $Id: DumbDraw.c,v 1.9 1995/07/28 23:35:17 dhb Exp $ */
-/* $Log: DumbDraw.c,v $
+/* $Id: DumbDraw.c,v 1.12 2000/06/12 04:28:17 mhucka Exp $ */
+/*
+ * $Log: DumbDraw.c,v $
+ * Revision 1.12  2000/06/12 04:28:17  mhucka
+ * Removed nested comments within header, to make compilers happy.
+ *
+ * Revision 1.11  2000/05/02 06:06:41  mhucka
+ * Added type casts for certain function call arguments.
+ *
+ * Revision 1.10  1998/07/15 06:17:26  dhb
+ * Upi update
+ *
  * Revision 1.9  1995/07/28 23:35:17  dhb
  * Undefine __OPTIMIZE__ macro for gcc which causes incompatible
  * redefinition of functions in stdlib.h.
@@ -44,6 +54,9 @@
 #define __OPTIMIZE__
 #endif /* __GNUC__ && __OPTIMIZE__ */
 #endif
+#if defined(__GNUC__) && defined(decalpha)
+#include <stdlib.h>
+#endif /* decalpha */
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
 #include "Xo/XoDefs.h"
@@ -261,7 +274,7 @@ static void Resize(w)
   int num_children = dw->composite.num_children;
   PixObjectClass class;
 
-	if (!XtIsRealized(dw))
+	if (!XtIsRealized((Widget) dw))
 		return;
 
 	/* handle children */
@@ -287,7 +300,7 @@ static void Redisplay (w, e, region)
   int num_children = dw->composite.num_children;
 
 	if ((dw->coredraw.drawflags & XO_DRAW_REFRESHABLE_NOT) ||
-		!XtIsRealized(dw))
+		!XtIsRealized((Widget) dw))
 		return;
 
 	/* handle children */
