@@ -1,5 +1,15 @@
-/* $Id: Plot.c,v 1.12 1996/08/10 00:53:57 venkat Exp $ */
-/* $Log: Plot.c,v $
+/* $Id: Plot.c,v 1.15 2000/10/09 22:59:32 mhucka Exp $ */
+/*
+ * $Log: Plot.c,v $
+ * Revision 1.15  2000/10/09 22:59:32  mhucka
+ * Removed needless declaration for sqrt().
+ *
+ * Revision 1.14  2000/06/12 04:28:17  mhucka
+ * Removed nested comments within header, to make compilers happy.
+ *
+ * Revision 1.13  2000/05/02 06:06:42  mhucka
+ * Added type casts for certain function call arguments.
+ *
  * Revision 1.12  1996/08/10 00:53:57  venkat
  * Changed SetValues to reset the plot range resources (xmaxvalue ...) if
  * the number of points == 0. The old values were retained earlier and
@@ -360,7 +370,7 @@ static Boolean SetValues (curw, reqw, neww)
 				if (news->plot.topts == curs->plot.alloced_pts)
 					news->plot.topts = news->plot.alloced_pts;
 				news->plot.pts =
-					(XPoint *)XtRealloc(news->plot.pts,
+					(XPoint *)XtRealloc((char *) news->plot.pts,
 						news->plot.alloced_pts*sizeof(XPoint));
 			}
 		}
@@ -463,9 +473,6 @@ static void Project (w)
   double ty = sw->pix.ty;
   double tz = sw->pix.tz;
   CoreDrawWidgetClass class = (CoreDrawWidgetClass)XtClass(XtParent(w));
-#ifndef __PARAGON__
-  double sqrt();
-#endif
   int parentwid= ((CoreDrawWidget)XtParent(w))->core.width;
   int parentht= ((CoreDrawWidget)XtParent(w))->core.height;
   int	*px,*py,*pz;
@@ -573,12 +580,12 @@ static void Project (w)
 	sw->plot.yminvalue = yminvalue;
 	sw->plot.ymaxvalue = ymaxvalue;
 	
-  XtFree(px);
-  XtFree(py);
-  XtFree(pz);
-  XtFree(sxpts);
-  XtFree(sypts);
-  XtFree(szpts);
+  XtFree((char *) px);
+  XtFree((char *) py);
+  XtFree((char *) pz);
+  XtFree((char *) sxpts);
+  XtFree((char *) sypts);
+  XtFree((char *) szpts);
 }
 
 
@@ -618,7 +625,7 @@ XoAddPtsToPlot(w,newnpts)
 			sw->plot.topts = apts;
 		sw->plot.alloced_pts = apts;
 		sw->plot.pts =
-			(XPoint *)XtRealloc(sw->plot.pts,
+			(XPoint *)XtRealloc((char *) sw->plot.pts,
 				apts*sizeof(XPoint));
 	}
 
