@@ -1,5 +1,16 @@
-/* $Id: XoColor.c,v 1.27 1997/07/25 00:38:19 venkat Exp $ */
-/* $Log: XoColor.c,v $
+/* $Id: XoColor.c,v 1.30 2001/06/29 21:10:47 mhucka Exp $ */
+/*
+ * $Log: XoColor.c,v $
+ * Revision 1.30  2001/06/29 21:10:47  mhucka
+ * Added some code to support the ability to set a default colormap in
+ * TopLevel.c
+ *
+ * Revision 1.29  2000/09/11 15:48:36  mhucka
+ * Used %d to %ld in one of the calls to fprintf involving a long int.
+ *
+ * Revision 1.28  2000/06/12 04:28:20  mhucka
+ * Removed nested comments within header, to make compilers happy.
+ *
  * Revision 1.27  1997/07/25 00:38:19  venkat
  * Fixed bugs to set variables before they were used in XoInitColorscale() and
  * XoGetPixelFromString()
@@ -141,6 +152,7 @@ static Colormap	usedcmap;
 static int	useddepth;
 static unsigned long colorscale[XO_NCOLORS+BUFFER];
 static int colorscale_alloced = 0;
+static int have_genesis_colormap = 0;
 static Boolean readonlyvisual=False;
 static Boolean colorscalesupport=True;
 static char  *visualname[] = {
@@ -189,8 +201,9 @@ Boolean XoColorSupported()
 **
 */
 
-XoMakeCmap(mapname)
+XoMakeCmap(mapname, reportmissingfile)
 	char	*mapname;
+	int	 reportmissingfile;
 {
 	extern int 	GENOPT_UseDefaultColor;
 	XColor		rgb[256],color[XO_NCOLORS],col1,col2;
@@ -214,8 +227,9 @@ XoMakeCmap(mapname)
 
 	fp = SearchForNonScript(mapname,"r");
 	if (fp == NULL) {
-		fprintf(stderr,"colormap file '%s' not found\n",mapname);
-		return;
+	        if (!reportmissingfile)
+		        fprintf(stderr,"colormap file '%s' not found\n", mapname);
+	        return;
 	}
 	for (i = 0 ; i < 256 ; i++)  {
 		if (!fscanf(fp,"%d%d%d",&r,&g,&b))
@@ -245,7 +259,9 @@ XoMakeCmap(mapname)
              cannot be created for such visuals**/
 
 	      if (XAllocColor(XgDisplay(),default_cmap,&color[i]) == 0) {
-			fprintf(stderr,"Cannot allocate any more colors at i = %d. Try closing other applications for better color handling \n",i);
+			fprintf(stderr,
+				"Cannot allocate any more colors at i = %d."
+				"Try closing other applications for better color handling \n",i);
 	       return;
 	      }
 	      else{
@@ -263,6 +279,12 @@ XoMakeCmap(mapname)
 
 	 }
 	colorscale_alloced = 1;
+	have_genesis_colormap = 1;
+}
+
+int XoHaveCreatedGenesisCmap()
+{
+    return have_genesis_colormap;
 }
 
 XoInitColorscale()
@@ -472,7 +494,7 @@ XoInitColorscale()
     XFree(depthlist);
 #ifdef XOCOLOR_DEBUG
  fprintf(stderr, "The visual used is %s\n",visualname[usedvisual->class]);
- fprintf(stderr, "The colormap used is %d\n",(int)usedcmap);
+ fprintf(stderr, "The colormap used is %ld\n",(int)usedcmap);
  fprintf(stderr, "The used depth is %d\n", useddepth);
 #endif
 }
