@@ -1,5 +1,21 @@
-/* $Id: xshape.c,v 1.18 1997/07/18 23:55:56 dhb Exp $ */
-/* $Log: xshape.c,v $
+/* $Id: xshape.c,v 1.22 2000/06/12 04:20:46 mhucka Exp $ */
+/*
+ * $Log: xshape.c,v $
+ * Revision 1.22  2000/06/12 04:20:46  mhucka
+ * 1) Removed nested comments in RCS/CVS log lines, to quiet down the
+ *    IRIX cc compiler.
+ * 2) Added NOTREACHED comments where appropriate.
+ *
+ * Revision 1.21  2000/05/02 06:18:31  mhucka
+ * Added type casts for certain function call arguments.
+ *
+ * Revision 1.20  1999/10/17 04:02:32  mhucka
+ * Applied additional patches from Upi dated July 1998 that never seem to have
+ * made it into DR 2.2.1.
+ *
+ * Revision 1.19  1998/07/15 06:23:58  dhb
+ * Upi update
+ *
  * Revision 1.18  1997/07/18 23:55:56  dhb
  * Updated ShowInterpol() calls to add dimension arg
  *
@@ -163,7 +179,7 @@ XShape (shape, action)
 		XtNypts,(XtPointer)((Interpol *)shape->ypts)->table,
 		XtNzpts,(XtPointer)((Interpol *)shape->zpts)->table,
 		NULL);
-	XtAddCallback(shape->widget, XtNcallback,xoCallbackFn,
+	XtAddCallback((Widget) shape->widget, XtNcallback,xoCallbackFn,
 		(XtPointer)shape);
 
     ac--, av++;			/* object type */
@@ -219,6 +235,7 @@ XShape (shape, action)
 		if(localav[i] != NULL) free(localav[i]);
 	}
 	return(1);
+	/* NOTREACHED */
 	break;
   case COPY:
 	/* Note that the 'shape' pointer refers to the original,
@@ -260,11 +277,12 @@ XShape (shape, action)
 			NULL);
 		
 	/* adding the callback */
-		XtAddCallback(copy_shape->widget, XtNcallback,xoCallbackFn,
+		XtAddCallback((Widget) copy_shape->widget, XtNcallback,xoCallbackFn,
 			(XtPointer)copy_shape);
 	/* Converting the rest of the values */
 		gx_convert_all(copy_shape,GXconvert,XtNumber(GXconvert));
 	return(1);
+	/* NOTREACHED */
 	break;
   case SET:
 	if (xoSetPixFlags(shape,ac,av)) return(1);
@@ -365,7 +383,7 @@ XShape (shape, action)
 					(shape->npts+20) * sizeof(double));
 				ip = (Interpol *) shape->xpts;
 				xomemzero((char*) &ip->table[shape->allocated_pts], (ap - shape->allocated_pts)*sizeof(double));
-				((Interpol *)(shape->xpts))->xdivs = shape->npts+20;
+				((Interpol *)(shape->xpts))->xdivs = shape->npts + 20;
 
 				((Interpol *)(shape->ypts))->table = (double *) 
 					realloc(((Interpol *)(shape->ypts))->table,
@@ -404,6 +422,7 @@ XShape (shape, action)
 		if (ShowInterpol(action,"xpts",shape->xpts,1,100)) return(1);
 		if (ShowInterpol(action,"ypts",shape->ypts,1,100)) return(1);
 		return(ShowInterpol(action,"zpts",shape->zpts,1,100));
+	/* NOTREACHED */
 	break;
   case XSHAPE_ADDPT: /* adds a point to the arrays */
 	if (ac < 3) break;
