@@ -1,5 +1,24 @@
-/* $Id: xtree.c,v 1.30 1997/07/16 21:33:09 dhb Exp $ */
-/* $Log: xtree.c,v $
+/* $Id: xtree.c,v 1.35 2001/06/29 21:11:58 mhucka Exp $ */
+/*
+ * $Log: xtree.c,v $
+ * Revision 1.35  2001/06/29 21:11:58  mhucka
+ * Added extra parens inside conditionals and { } groupings to quiet certain
+ * compiler warnings.
+ *
+ * Revision 1.34  2001/04/18 22:39:35  mhucka
+ * Miscellaneous portability fixes, mainly for SGI IRIX.
+ *
+ * Revision 1.33  2000/06/12 04:25:42  mhucka
+ * 1) Removed nested comments in RCS/CVS log lines, to quiet down the
+ *    IRIX cc compiler.
+ * 2) Added NOTREACHED comments where appropriate.
+ *
+ * Revision 1.32  2000/05/02 06:18:31  mhucka
+ * Added type casts for certain function call arguments.
+ *
+ * Revision 1.31  1998/07/15 06:23:58  dhb
+ * Upi update
+ *
  * Revision 1.30  1997/07/16 21:33:09  dhb
  * Change getopt() calls to avoid conflict with unix version.
  *
@@ -168,7 +187,8 @@
 #include "Draw/Tree.h"
 #include "draw_ext.h"
 #include "widg/xform_struct.h"
-#include "../Xo/XoDefs.h"
+#include "Xo/XoDefs.h"
+#include "Xo/Xo_ext.h"
 #include "hash.h"
 
 #define XO_TREE_MAXSHAPES 100
@@ -311,7 +331,7 @@ XTree (tree, action)
 		XtNcolorscale, (XtPointer)XoColorscale(),
 		XtNncols,(int)XoNcolors(),
 		NULL);
-	XtAddCallback(tree->widget, XtNcallback,xoCallbackFn,
+	XtAddCallback((Widget) tree->widget, XtNcallback,xoCallbackFn,
 		(XtPointer)tree);
 	AddMsgArrow(tree,NULL,NULL,NULL,NULL,NULL,0,NULL,NULL,NULL,NULL);
 
@@ -337,6 +357,7 @@ XTree (tree, action)
 	newaction.argv[11] = "c";
 	elm = Create("xshape","shape",tree,&newaction,0);
 	return(1);
+	/* NOTREACHED */
 	break;
   case COPY: /* currently incomplete */
 	/* Note that the 'tree' pointer refers to the original,
@@ -361,11 +382,12 @@ XTree (tree, action)
 */
 		
 	/* adding the callback */
-		XtAddCallback(copy_tree->widget, XtNcallback,xoCallbackFn,
+		XtAddCallback((Widget) copy_tree->widget, XtNcallback,xoCallbackFn,
 			(XtPointer)copy_tree);
 	/* Converting the rest of the values */
 		gx_convert_all(copy_tree,GXconvert,XtNumber(GXconvert));
 		return(1);
+		/* NOTREACHED */
 	break;
   case SET:
 	if (xoSetPixFlags(tree,ac,av)) return(1);
@@ -599,7 +621,7 @@ XTree (tree, action)
 		printoptusage(ac,av);
 		return(0);
 	}
-	if (elm = GetElement(optargv[1])) {
+	if ((elm = GetElement(optargv[1]))) {
 		int i;
 		float x, y, z;
 		float inx = Atof(optargv[2]);
@@ -631,6 +653,7 @@ XTree (tree, action)
 		return(1);
 	}
 	return(0);
+	/* NOTREACHED */
 	break;
   case TRUNCATE : /* elmname [mode] */
 	/* shuffle so that we have the calling funcname too */
@@ -893,7 +916,7 @@ static int TreeLayout(tree)
 	for(i = 0 ; i < nelms; i++) {
 		elm = tree->coord_elist->element[i];
 		orig[i].child = orig[i].next = -1;
-		if (parent = elm->parent) { /* root element */
+		if ((parent = elm->parent)) { /* root element */
 			int elmno = FindElistIndex(tree,parent);
 			if (elmno == XO_ELM_NOT_FOUND) {
 				orig[i].next = root.child;
@@ -1206,14 +1229,14 @@ static void FindMsgArrows(tree)
 	/* scanning the msgdest list for all msgs that meet criteria */
 		lastobject = NULL;
 		for(i = 0 ; i < msgdest_elist->nelements; i++) {
-			int elmindex;
+			ADDR elmindex;
 			Element *elm = msgdest_elist->element[i];
 			if (!elm) continue;
 
 			/* find elm index */
 			tempentry = xo_hash_find(elm,hash_table);
 			if (!tempentry) continue;
-			elmindex = (int)tempentry->data;
+			elmindex = (ADDR)tempentry->data;
 
 			/* identify msg type */
 			/* Skip extra lookups if object type is same as last */
@@ -1602,7 +1625,7 @@ static int BuildTreeList(tree,ncoords)
 		if (tree->coord_elist && tree->ncoords > 0)
 			FreeElementList(tree->coord_elist);
 			tree->coord_elist = NULL;
-		if (coord_hash = (HASH *)tree->coord_hash) {
+		if ((coord_hash = (HASH *)tree->coord_hash)) {
 			if (coord_hash->size > 0)
 				free(coord_hash->entry);
 			free(tree->coord_hash);
@@ -1623,7 +1646,7 @@ static int BuildTreeList(tree,ncoords)
 				if (strcmp(tree->path,"all") == 0) {
 					/* count the objects */
 					for(i = 0; i < object_hash_table->size; i++)
-						if (name = object_hash_table->entry[i].key)
+					        if ((name = object_hash_table->entry[i].key))
 							nobj++;
 					tree->coord_elist = (ElementList *)
 						calloc(1,sizeof(ElementList));
@@ -1632,7 +1655,7 @@ static int BuildTreeList(tree,ncoords)
 						calloc(nobj,sizeof(Element *));
 					nobj = 0;
 					for(i = 0; i < object_hash_table->size; i++) {
-						if (name = object_hash_table->entry[i].key) {
+						if ((name = object_hash_table->entry[i].key)) {
 							obj = (GenesisObject *)
 								object_hash_table->entry[i].data;
 							tree->coord_elist->element[nobj] =
@@ -1647,7 +1670,7 @@ static int BuildTreeList(tree,ncoords)
 
 					StringToArgList(tree->path,&argc,&argv,0);
 					for(i = 0; i < argc; i++) {
-						if (obj=GetObject(argv[i]))
+					        if ((obj=GetObject(argv[i])))
 							nobj++;
 					}
 					tree->coord_elist = (ElementList *)
@@ -1658,7 +1681,7 @@ static int BuildTreeList(tree,ncoords)
 						calloc(nobj,sizeof(Element *));
 					nobj = 0;
 					for(i = 0; i < argc; i++) {
-						if (obj=GetObject(argv[i])) {
+						if ((obj=GetObject(argv[i]))) {
 							tree->coord_elist->element[nobj] =
 								obj->defaults;
 							nobj++;
@@ -1675,7 +1698,9 @@ static int BuildTreeList(tree,ncoords)
 		/* Set up the elists */
 				tree->coord_elist = WildcardGetElement(tree->path,0);
 				if (tree->coord_elist->nelements==0) {
+				/*
 					printf("Warning: No elements found on path %s\n", tree->path);
+					*/
 				if (tree->path[0] != '/')
 					printf("Note: Use absolute path names for the path field\n");
 					tree->ncoords = 0;
