@@ -1,7 +1,14 @@
-static char rcsid[] = "$Id: mapcommand.c,v 1.8 1995/08/09 17:13:40 dhb Exp $";
+static char rcsid[] = "$Id: mapcommand.c,v 1.10 2001/06/29 21:13:11 mhucka Exp $";
 
 /*
 ** $Log: mapcommand.c,v $
+** Revision 1.10  2001/06/29 21:13:11  mhucka
+** Added extra parens inside conditionals and { } groupings to quiet certain
+** compiler warnings.
+**
+** Revision 1.9  2000/06/12 04:33:00  mhucka
+** Added a type specifier where one was missing.
+**
 ** Revision 1.8  1995/08/09 17:13:40  dhb
 ** Changed mapping of xaddpts to use the ADDPTS action on the xplot
 ** rather than the xaddpts X1compat script function.  The -color option
@@ -508,11 +515,12 @@ char*	argv[];
 	    SepGeom(widgets, &xwidget, &ywidget, &wwidget, &hwidget);
 	    
 	if (xwidget != NULL || ywidget != NULL ||
-	    wwidget != NULL || hwidget != NULL)
+	    wwidget != NULL || hwidget != NULL) {
 	    if (geom != NULL && geom[0] == '[')
 		geom = TransformGeom(geom,xwidget,ywidget,wwidget,hwidget);
 	    else
 		geom = TransformGeom("[20,20,200,30]",xwidget,ywidget,wwidget,hwidget);
+	}
 
 	if (coords[0] != '\0')
 	  {
@@ -751,7 +759,7 @@ char*	argv[];
 static WarnAboutEnvVars()
 
 {
-	static warned_already = 0;
+	static int warned_already = 0;
 
 	if (!warned_already)
 	  {
@@ -1475,11 +1483,12 @@ char*	argv[];
 	for (nxtarg = 0; argv[nxtarg] != NULL; nxtarg++)
 	    /* count the args */;
 
-	if (nxtarg == 8)
+	if (nxtarg == 8) {
 	    if (argv[7][0] == 'd')
 		argv[7] = "-duplicate";
 	    else
 		argv[7] = NULL;
+	}
 
 	return 1;
 }
