@@ -1,7 +1,10 @@
-static char rcsid[] = "$Id: strsave.c,v 1.2 1997/08/08 19:28:43 dhb Exp $";
+static char rcsid[] = "$Id: strsave.c,v 1.3 2000/07/12 06:11:55 mhucka Exp $";
 
 /*
 ** $Log: strsave.c,v $
+** Revision 1.3  2000/07/12 06:11:55  mhucka
+** Added #include of string.h and malloc.h.
+**
 ** Revision 1.2  1997/08/08 19:28:43  dhb
 ** Added include of stdlib.h for malloc() return type
 **
@@ -18,6 +21,8 @@ static char rcsid[] = "$Id: strsave.c,v 1.2 1997/08/08 19:28:43 dhb Exp $";
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <malloc.h>
+#include <string.h>
 
 extern int errno;
 
