@@ -1,18 +1,52 @@
 /*********************************************************************
 
-** This library uses the netcdf - version 2.4.3 which is
+** This library uses the netcdf - version 3.4 which is
 ** Copyright 1993-1997 University Corporation for Atmospheric Research/Unidata
 
 ** The netcdf library is provided as per the terms of the
-** UCAR/Unidata license
+** UCAR/Unidata license.
 
-** See interface/netcdf-2.4.3/COPYRIGHT for the full notice
+** See interface/netcdf-3.4/copyright.html for the full notice.
 
 ********************************************************************/
 
-/* $Id: gen_diskio.c,v 1.3 1997/08/04 04:18:19 dhb Exp $ */
-/* $Log: gen_diskio.c,v $
- * Revision 1.3  1997/08/04 04:18:19  dhb
+/* $Id: gen_diskio.c,v 1.12 2001/06/29 21:13:24 mhucka Exp $ */
+/*
+ * $Log: gen_diskio.c,v $
+ * Revision 1.12  2001/06/29 21:13:24  mhucka
+ * Added extra parens inside conditionals and { } groupings to quiet certain
+ * compiler warnings.
+ *
+ * Revision 1.11  2001/04/18 22:39:36  mhucka
+ * Miscellaneous portability fixes, mainly for SGI IRIX.
+ *
+ * Revision 1.10  2000/06/12 05:07:01  mhucka
+ * Removed nested comments; added NOTREACHED comments where appropriate.
+ *
+ * Revision 1.9  1999/10/16 21:52:46  mhucka
+ * Merged Upi's changes for supporting netCDF 3.4.  The changes in most files
+ * only involve the copyright notice.  I also fixed up the comments slightly
+ * to reflect the location of the actual netCDF copyright file.
+ *
+ * Revision 1.8  1999/10/16 21:23:47  mhucka
+ * Venkat had a version of the diskio code in his home directory that he
+ * appears to have been working on, and that did not make it into the
+ * source tree that I had originally used as the starting point of my
+ * GENESIS 2.2.2 DR reorganization effort.  In order to merge Venkat's
+ * changes into the distribution, I'm reverting to his version, then will
+ * update his version to incorporate recent changes by Upi for netCDF 3.4.
+ *
+ * Revision 1.6  1998/01/15 02:19:50  venkat
+ * Inclusion-of-interface-headers-in-subdirectory-changed-to-be-not-hardcoded.-Now-this-is-taken-care-through-the-use-of-the-'-I'-flag-during-compilation-in-the-Makefile
+ *
+ * Revision 1.5  1998/01/14 00:38:32  venkat
+ * Lots of changes to accomodate support for FMT1.
+ *
+ * Revision 1.4  1997/10/07 22:26:55  venkat
+ * RESET action now creates the start_time and dt as global metadata children.
+ * These can be used to obtain the header information.
+ *
+ * Revision 1.3  1997/08/04  04:18:19  dhb
  * include of strings.h does not work under Solaris.  Include
  * of string.h should suffice.
  *
@@ -26,7 +60,7 @@
 #include <sys/types.h>
 
 
-#include "interface/absff_ext.h"
+#include "absff_ext.h"
 #include "diskio_ext.h"
 
 #include <sys/time.h>
@@ -34,7 +68,7 @@
 #include <time.h>
 
 #ifndef NSUPPORTED_FORMATS
-#define NSUPPORTED_FORMATS 1
+#define NSUPPORTED_FORMATS 2
 
 #ifndef INVALID_FORMAT
 #define INVALID_FORMAT -999
@@ -48,11 +82,13 @@ static int ffAllocateFrame(/*struct Gen_Diskio_Type * */);
 static int ffReadFrame(/*struct Gen_Diskio_Type * , long begin_index[]*/);
 static int ffGet_autoSAVE_diskio_Framesize(/*struct Gen_Diskio_Type *, int *datasize*/);
 static int ffSetUpInterpols(/*struct Gen_Diskio_Type * */);
+static int ffCreateDefaultGlobalMetadata(/* struct Gen_Diskio_Type * */);
 
 static Element * ffGetChild(/* Element *parent, char *objectname, char *childname, int childindex */);
 
 static FormatName2Number_Table name2number[] = {
-	{"netcdf", 1} 
+	{"FMT1", 1}, 
+	{"netcdf", 2}
 };
 
 static char * Make_System_Info_String(sysinf)
@@ -86,14 +122,12 @@ static char * Make_System_Info_String(sysinf)
  return localbuf;
 }
 
- Abs_Diskio* ((*ConstructDiskio_Interface[NSUPPORTED_FORMATS])(/* char *filename, FileAccessMode mode */)) = {
-	Netcdf_ConstructFile
- };
 
  Gen_Diskio(diskio, action) 
   register struct Gen_Diskio_Type *diskio;
   Action *action;
 {
+  char *SelfMetadata_Interface();
   char *filename;
   char *formatname;
   char mode;
@@ -105,7 +139,7 @@ static char * Make_System_Info_String(sysinf)
   char buf[BUFSIZ];
   char localbuf[BUFSIZ];
   char timebuf[BUFSIZ];
-  float currenttime;
+  float currenttime, fval;
   char *vals[1];
 
   long count[1];
@@ -120,7 +154,6 @@ static char * Make_System_Info_String(sysinf)
   MsgIn *msg;
   static int messages_changed = 0;
   static int messages_deleted = 0;
-  
 
   char **av = action->argv;
   int ac = action->argc;
@@ -154,27 +187,13 @@ static char * Make_System_Info_String(sysinf)
 				if(strcmp((av[i]+1), "accessmode") == 0){
 					mode = av[i+1][0]; 
 					if (mode == 'a') {
-						ffError("Mode '%c' is not yet supported\n", mode);
+						ffError("Mode '%c' is not yet supported \n", mode);
 						fprintf(stderr, "Defaulting to read mode \n");
 						mode = 'r';
 					}
 				}
 			}
-			if(formatname == NULL) formatname = CopyString("netcdf");
-			nname2number = FF_Number(name2number);
-			for(i=0; i<nname2number; i++) {
-				if(strcasecmp(formatname, name2number[i].name) == 0) {
-					format = name2number[i].number - 1;
-					break;
-				}
-			}
-			if(i == nname2number) format = INVALID_FORMAT;
-			if(format == INVALID_FORMAT){
-				fprintf(stderr, "The format '%s' is not currently supported\n", formatname);
-				FreeString(formatname);
-				if(filename != NULL) FreeString(filename);
-				return 0;
-			}
+			if(formatname == NULL) formatname = CopyString(" ");
 			if(filename == NULL) filename = CopyString(diskio->name);
 
 		/* 
@@ -186,12 +205,11 @@ static char * Make_System_Info_String(sysinf)
 		}
 
 
-
-		diskio->adi = (char *) ConstructDiskio_Interface[format](filename, mode);
+		diskio->adi = (char *) Absff_CreateDiskio_Interface(filename, mode, &formatname); 
 		adi = (Abs_Diskio *) diskio->adi;
 
 			
-		{	
+		{
 		/* 
 		###########################################################
 			BEGIN-SECTION
@@ -226,7 +244,6 @@ static char * Make_System_Info_String(sysinf)
 			diskio->is_open = adi->is_open;
 			diskio->is_writable = adi->is_writable;
 			diskio->accessmode = mode;
-
 			
 				
 			switch(mode){
@@ -249,44 +266,39 @@ static char * Make_System_Info_String(sysinf)
 
 				/** FIRST - ############################################## */
 
-				/** The 'Simulation_Clockrate' metadata element **/
+				/** The 'start_time' metadata element **/
 				j = 0;	
 				newargv[j] = "metadata"; j++;
-				newargv[j] = "Simulation_Clockrate"; j++;
+				newargv[j] = "start_time"; j++;
 
 				newargv[j] = "-values"; j++
-				/* NOTE: This may be incorrect */
 				sprintf(timebuf, "%f", SimulationTime());
 				newargv[j] = timebuf; j++;
 
-				childsize = 1; /* Only one float value */
-				sprintf(buf, "%d", childsize);
 				newargv[j] = "-size"; j++;
-				newargv[j] = buf; j++;
+				newargv[j] = "1"; j++;
 
 				newargv[j] = "-type"; j++;
 				newargv[j] = "float32"; j++;
 
 				newaction.argc = j;
-				Create("metadata", "Simulation_Clockrate", diskio, &newaction, 0);
+				Create("metadata", "start_time", diskio, &newaction, 0);
 
 				/** ############################################## */
 
 				/** SECOND - ############################################## */
 
-				/** The 'Sampling_Rate' metadata element **/
+				/** The 'dt' metadata element **/
 				j = 0;	
 				newargv[j] = "metadata"; j++;
-				newargv[j] = "Sampling_Rate"; j++;
+				newargv[j] = "dt"; j++;
 
 				newargv[j] = "-values"; j++
 				sprintf(timebuf, "%f", Clockrate(diskio));
 				newargv[j] = timebuf; j++;
 
-				childsize = 1; /* Only one float value */
-				sprintf(buf, "%d", childsize);
 				newargv[j] = "-size"; j++;
-				newargv[j] = buf; j++;
+				newargv[j] = "1"; j++;
 
 				newargv[j] = "-type"; j++;
 				newargv[j] = "float32"; j++;
@@ -358,9 +370,9 @@ static char * Make_System_Info_String(sysinf)
 		} /* adi != NULL */	
 		else {
 			if(mode == 'r') 
-				fprintf(stderr, "File '%s' not found\n", filename);
+				fprintf(stderr, "\nFile '%s' not found\n", filename);
 			else
-				fprintf(stderr, "Check permissions and/or pathname for '%s'\n", filename);	
+				fprintf(stderr, "\n\nCheck permissions and/or pathname for '%s'.\nAlso please check that the top-level Makefile macro settings \nfor compilation and linking of the '%s' libraries is uncommented.\n\n", filename, formatname);
 			FreeString(filename);
 			FreeString(formatname);
 		}
@@ -374,6 +386,7 @@ static char * Make_System_Info_String(sysinf)
 		}
 
 		return 0;
+		/* NOTREACHED */
 	break;
 
 	case SET:
@@ -391,19 +404,30 @@ static char * Make_System_Info_String(sysinf)
 			if((strcmp(field, "filename") == 0) && 
 			   (strcmp(value, diskio->filename) != 0)) {
 				Element *child, *nextchild;
+
+
+				/* Destruct the children, interfaces and all */
 				for(child = diskio->child; child != NULL; child=nextchild) {
 					nextchild = child->next;
 					DeleteElement(child);	
 				}
-				if((adi != NULL) && (adi->SetFile(adi, value, diskio->accessmode) != -1)){
+				if(coordi != NULL) {
+					coordi->Destruct(coordi);
+					free(coordi);	
+					diskio->coordi = NULL;
+				}
+				adi->DestructChildInterfaces(adi);
+				adi->Destruct(adi);
+				free(adi);
+
+				/* Recreate the interfaces with the new filename */
+				diskio->adi = (char *)Absff_CreateDiskio_Interface(value, diskio->accessmode, &diskio->fileformat);
+				adi = (Abs_Diskio *) diskio->adi;
+				if(adi != NULL) { 
 					FreeString(diskio->filename); 
 					diskio->filename = CopyString(value);
 
 					/* Reconstruct the default "SimulationTime" coordi interface */
-					if(coordi != NULL) {
-						coordi->Destruct(coordi);
-						free(coordi);	
-					}
 					diskio->coordi = (char *) adi->ConstructCoord_Interface(adi, "SimulationTime");
 
 					/* Recreate the new hierarchy */
@@ -422,6 +446,7 @@ static char * Make_System_Info_String(sysinf)
 					fprintf(stderr, "Error setting field '%s' on object '%s'\n", 
 							field, diskio->name);
 				}
+				return 1;
 			}
 			
 			if (strcmp(field, "fileformat") == 0) {
@@ -458,11 +483,17 @@ static char * Make_System_Info_String(sysinf)
 						}
 					}
 				}	
+				return 1;
+			}
+
+			if (strcmp(field, "append") == 0) {
+  				short appendmode;
+				appendmode = atoi(value); 
+				adi->SetAppendMode(adi, appendmode);	 
 			}
-			/* Disallow setting of field by base code in all cases*/
-			return 1;
 		}
 	return 0;
+	/* NOTREACHED */
 	break;
 
 	case ADDMSGIN:
@@ -478,68 +509,48 @@ static char * Make_System_Info_String(sysinf)
 		   perspective .  */	
 
 		/* 
-		   Now Moved to the RESET action since this stuff needs to be 
+		   The above functionality moved to the RESET action 
+		   since this stuff needs to be 
 		   done there anyway to accomodate any additions/deletions 	
 		   of messages 
-
-		if((MsgIn *) (action->data) == NULL)
-			break;
-		if (! ffCreateMetadataToForwardMsg(diskio, (MsgIn*)(action->data)))
-			return 0;	
-		return ffCreateVarToForwardMsg(diskio, (MsgIn*)(action->data));
-
 		*/
-		/* 
+
+	/* 
 			The static variable (to this function) "messages_changed" 
 			is updated here and/or if a message is deleted so things can 
 			be optimized a bit in the RESET action - The creation of 
 			the implicit variables and metadata children takes place
 			conditionally 
 		*/
-		msg = (MsgIn*) (action->data);
-		if(msg->type == 0) /* SAVE */
-			messages_changed = 1;
+		
+		/* For FMT1 we cannot add messages if RESET was called already */
+		if (strcmp(diskio->fileformat, "FMT1") != 0 || !diskio->already_reset) {
+			msg = (MsgIn*) (action->data);
+			if(msg->type == 0) /* SAVE */
+				messages_changed = 1;
+		} else {
+			fprintf(stderr, "Messages cannot be added to diskio element '%s' supporting fileformat '%s' after being reset.\n", diskio->name, diskio->fileformat);
+		}
+		
 	break;
 
 	case DELETEMSGIN:
-		/*
-		msg = (MsgIn*) (action->data);
-		if(msg != NULL) {
-			int newindex, lastindex, deleted;
-			Element *child, *nextchild;
-			newindex = ffDetermineNewIndex(diskio, "auto3-DPosSAVE_diskio");
-			lastindex = newindex - 1;
-			for(child=diskio->child; child != NULL; child=nextchild) {
-				nextchild=child->next;
-				deleted = 0;
-				if ((strcmp(BaseObject(child)->name, "metadata")==0) &&
-					(strcmp(child->name, "auto3-DPosSAVE_diskio") == 0) &&
-						(child->index == lastindex)) {
-					DeleteForwMsg(child, msg);	
-					DeleteElement(child);
-					deleted = 1;
-				}
-				if(!deleted && 
-					((strcmp(BaseObject(child)->name, "variable")==0) &&
-                                       		(strcmp(child->name, "autoSAVE_diskio") == 0) &&
-                                               		(child->index == lastindex))) {
-					DeleteForwMsg(child, msg);
-					DeleteElement(child);
-				}
-					
-			}
-		}	
-		*/
-		msg = (MsgIn*) (action->data);
-		if(msg->type == 0) { /* SAVE */
+		if (strcmp(diskio->fileformat, "FMT1") != 0 || !diskio->already_reset) {
+			msg = (MsgIn*) (action->data);
+			if(msg->type == 0) { /* SAVE */
 		/* The static "messages_deleted" variable reference counts the 
 		   number of deleted messages so the naming of the automatic variables 
 		   and metadata can follow a policy of reuse. The deleted message data
 		   will still exist in the file but will be flagged as inactive by 
 		   a simple renaming of the element 
 		*/
-			messages_deleted++;	
-			messages_changed = 1;
+				messages_deleted++;	
+				messages_changed = 1;
+			}
+		} else {
+		/*
+			fprintf(stderr, "Messages cannot be deleted from diskio element '%s' supporting fileformat '%s' after being reset.\n", diskio->name, diskio->fileformat);
+		*/
 		}
 	break;
 
@@ -547,8 +558,7 @@ static char * Make_System_Info_String(sysinf)
 		adi = (Abs_Diskio *) diskio->adi;
 		coordi = (Abs_Variable *) diskio->coordi;
 
-        	currenttime = SimulationTime();
-        	sprintf(buf, "%g", currenttime);
+        	sprintf(buf, "%g", diskio->currenttime);
 		switch(diskio->accessmode){
 			case 'r':
 				if (diskio->nmsgout) {
@@ -558,6 +568,9 @@ static char * Make_System_Info_String(sysinf)
 
 					if (coordi != NULL)
 						frameindex[0] = coordi->FastGetIndexForInput(coordi, buf);
+					else  
+						/* If the coordi is NULL (e.g FMT1, we calculate the index sequentially based on the simulation time */ 
+						frameindex[0] = (long) (int)((simulation_time - diskio->time_offset)/diskio->dt);
 					if(ffReadFrame(diskio, frameindex) == -1) {
 						ffError("Cannot read values from '%s'\n", diskio->filename);					
 					}
@@ -566,22 +579,54 @@ static char * Make_System_Info_String(sysinf)
 
 			case 'w':
 			case 'a':
-				count[0] =1;
+				if(diskio->filename == NULL)
+					diskio->filename = CopyString(diskio->name);
+				count[0] =1L;
+				if (adi != NULL) {
+					if (strcmp(diskio->fileformat, "FMT1") == 0) {
+						int nframe = 0;
+
+						if (!diskio->leave_open) {
+							adi->CloseFile(adi); 
+							if (adi->OpenFile(adi, diskio->filename, 'a') == -1) {
+								diskio->is_open = adi->is_open;
+								diskio->is_writable = adi->is_writable;
+								return 0;
+							
+							}
+						}
+						MSGLOOP(diskio, msg) {
+							case 0: /* SAVE */
+							fval = MsgValue(msg, float, 0);
+							adi->SetVariableValues(adi, nframe, &fval, diskio->start_time_index,  count);
+							nframe++;
+							break;
+						}
+					}
+				}
 			 	/* Write out the coordinate variable values every time step */
         			vals[0] = buf;
         			/* UGLY_SIGNPOST - Need to find out the name of the coord before doing this */
 				if (coordi != NULL)
                 			coordi->SetValuesFromString(coordi, vals, diskio->start_time_index, count);
+				if (diskio->flush && adi != NULL)
+					adi->Flush(adi);
 			break;
 		}
 #if 0
 		diskio->start_time_index[0]+=round(Clockrate(diskio)/ClockValue(0));
 #endif
 		diskio->start_time_index[0]++;
+        	diskio->currenttime += Clockrate(diskio);
 		
 	break;
 
 	case RESET:
+		adi = (Abs_Diskio*) diskio->adi;
+		if (adi == NULL)
+			return 0;
+		coordi = (Abs_Variable *) diskio->coordi;
+
 		if(diskio->accessmode == 'r') {
 		/** 
 		    This section of code deals primarily with BC issues with 
@@ -599,19 +644,119 @@ static char * Make_System_Info_String(sysinf)
 		    starting from which values will be searched for the 
 		    appropriate frame of data to be read in, is also reset.
 		**/
-			coordi = (Abs_Variable *) diskio->coordi;
-			if (coordi != NULL)
+			if (coordi != NULL) {
+				long start_index[1], count[1];
+				char *vals[2];
+				start_index[0] = 0L; count[0] = 2L;
+				coordi->GetValuesAsString(coordi, vals, start_index, count);
+				diskio->start_time = Atof(vals[0]);
+				diskio->dt = Atof(vals[1]) - Atof(vals[0]);
+				FreeString(vals[0]);
+				FreeString(vals[1]);
+				/* 
+				  Get the datatype also from the SAVEavi variable interface
+				  diskio->datatype = ?? 
+				*/
+				diskio->datatype = FLOAT;
 				coordi->ResetFastIndexForInput(coordi);		
+			}
+			else {
+				/* All the FMT1 header info is implemented 
+				 now as diskio fields and not metadata */
+				diskio->start_time = adi->GetStartTime(adi);
+				diskio->dt = adi->GetDt(adi);
+				diskio->datatype = adi->GetDataType(adi);
+			}
+
+
 
 			if (! diskio->valsallocated) {
 				int status;
 				/* Note: This routine (ffAllocateFrame) sets the nx, ny fields */
-				if (status = ffAllocateFrame(diskio)) 
+				if ((status = ffAllocateFrame(diskio)))
 					status = ffSetUpInterpols(diskio);
 				if (status)
 					diskio->valsallocated = 1;
 			}
+			diskio->start_time_index[0] = 0;
 		} else if(diskio->accessmode == 'w') {
+			int framesize;	
+			Element *child, *nextchild;
+
+			if (!diskio->append) {
+				int deleted;
+				int otherchildrenexist = 0;
+				char openmode;
+				for(child = diskio->child; child != NULL; child=nextchild) {
+					deleted = 0;
+					nextchild = child->next;
+					if(strcmp(BaseObject(child)->name, "metadata")==0) {
+						if(strcmp(child->name, "auto3-DPosSAVE_diskio") == 0) {
+							DeleteElement(child);	
+							deleted = 1;
+						} else
+							otherchildrenexist = 1;
+					}
+					if(!deleted && strcmp(BaseObject(child)->name, "variable")==0) {
+						if(strcmp(child->name, "autoSAVE_diskio") == 0) {
+							DeleteElement(child);	
+						} else
+							otherchildrenexist = 1;
+					}
+				}
+				adi->DestructChildInterfaces(adi);
+				if(coordi != NULL) {
+					coordi->Destruct(coordi);
+					free(coordi);
+				}
+				if (diskio->is_open) {
+					adi->ResetFile(adi);
+					adi->CloseFile(adi);
+					diskio->is_open = diskio->is_writable = 0;
+				}
+				/* 
+				 If other children exist : Only possible in 
+				 the case of self-describing file formats 
+				 such as netcdf. 
+				
+				 In such a situation we avoid clobbering 
+				 up the file by reopening it in 'w' mode.
+				 We use 'a' mode instead.
+				*/
+
+				if (otherchildrenexist) 
+					openmode = 'a';
+				else
+					openmode = diskio->accessmode;
+				if(adi->OpenFile(adi, diskio->filename, openmode) != -1){
+
+					/* Reconstruct the default "SimulationTime" coordi interface */
+					diskio->is_open = adi->is_open;
+					diskio->is_writable = adi->is_writable;
+					diskio->coordi = (char *) adi->ConstructCoord_Interface(adi, "SimulationTime");
+
+					diskio->start_time_index[0] = 0;
+					messages_changed = 1;
+
+				}
+			}
+			diskio->start_time = SimulationTime();
+			diskio->dt = Clockrate(diskio);
+			diskio->datatype = FLOAT; /* should be user-settable */
+			framesize = 0;
+			MSGLOOP(diskio, msg) {
+
+				case 0: /* SAVE */
+					framesize++;
+				break;
+			}
+			adi->BufferLabel(adi, diskio->fileformat);
+			adi->BufferStartTime(adi, diskio->start_time);
+			adi->BufferDt(adi, diskio->dt);	
+			adi->BufferFrameSize(adi, framesize);
+			adi->BufferDatatype(adi, diskio->datatype);
+
+
 			while(messages_deleted) {
 				int newindex, lastindex, deleted;
 				Element *child, *nextchild;
@@ -640,9 +785,12 @@ static char * Make_System_Info_String(sysinf)
 				}
 				messages_deleted--;
 			}
+
 			if(messages_changed) {
 				Element *child, *nextchild;
 				int deleted;
+				register int k;
+				float coord[3];
 				for(child = diskio->child; child != NULL; child=nextchild) {
 					nextchild = child->next;
 					deleted = 0;
@@ -653,19 +801,34 @@ static char * Make_System_Info_String(sysinf)
 						}
 					}
 					if(!deleted && strcmp(BaseObject(child)->name, "variable")==0) {
-						if(strcmp(child->name, "autoSAVE_diskio") == 0)
+						if(strcmp(child->name, "autoSAVE_diskio") == 0) {
 							DeleteElement(child);	
+						}
 					}
 				}
+				framesize = 0;
 				MSGLOOP(diskio, msg) {
 
 					case 0: /* SAVE */
 						if (! ffCreateMetadataAndForwardMsg(diskio, msg))
 							return 0;	
+						if (strcmp(diskio->fileformat, "FMT1") == 0) {
+
+							coord[0] = msg->src->x;
+							coord[1] = msg->src->y;
+							coord[2] = msg->src->z;
+
+							adi->SetMetadataValues(adi, framesize, coord, 0L, (long) 3);
+
+						}
 						if (! ffCreateVarAndForwardMsg(diskio, msg))
 							return 0;
+
+						framesize++;
 					break;
 				}
+	
+
 				/* 
 				   HACK ? The newly created autoSAVE children need to be 
 				   rescheduled 
@@ -675,52 +838,65 @@ static char * Make_System_Info_String(sysinf)
 				messages_changed = 0;
 			}
 		}
-		diskio->start_time_index[0] = 0;
+		diskio->header_size = adi->GetHeaderSize(adi);
+
+		if (!diskio->already_reset)
+			diskio->already_reset = 1;
+
+		diskio->currenttime = 0.0;
 	break;
 
 	case SHOW:
 		if (ShowInterpol(action, "xpts", diskio->xpts, diskio->valsallocated, 50)) return 1;
 		if (ShowInterpol(action, "ypts", diskio->ypts,diskio->valsallocated,  50)) return 1;
 		return (ShowInterpol(action, "zpts", diskio->zpts,diskio->valsallocated,  50));
+		/* NOTREACHED */
 	break;
 	
 	case QUIT:
 		adi = (Abs_Diskio *) diskio->adi;
-		if(diskio->accessmode == 'w' && adi != NULL) {
+		if((diskio->accessmode == 'w'  || diskio->accessmode == 'a' )&& adi != NULL) {
 			Element *child, *nextchild;
 			int stat = -1;
 			static Action forwaction = {"UPDATE", UPDATE, NULL, 0, NULL, NULL};
+	
+			/* We write the variable followed by metadata for 
+			 the same reason explained in the RESET action*/
 			for(child=diskio->child; child != NULL; child=nextchild) {
-				nextchild=child->next;
 				CallActionFunc(child, &forwaction);
+				nextchild=child->next;
 			}
 			if (adi != NULL)
 				stat = adi->Flush(adi);
 			if(stat != -1) return 1;
 		}
 		return 0;
+		/* NOTREACHED */
 	break;
 
 	case UPDATE:
 	case FLUSH:
 		adi = (Abs_Diskio *) diskio->adi;
-		if(diskio->accessmode == 'w' && adi != NULL) {
+		if((diskio->accessmode == 'w' || diskio->accessmode == 'a')&& adi != NULL) {
 			Element *child, *nextchild;
 			int stat = -1;
 			for(child=diskio->child; child != NULL; child=nextchild) {
-				nextchild=child->next;
 				CallActionFunc(child, action);
+				nextchild=child->next;
 			}
 			if (adi != NULL)
 				stat = adi->Flush(adi);
 			if(stat != -1) return 1;
 		}
 		return 0;
+		/* NOTREACHED */
 	break;
 
 	case DELETE:
 		adi = (Abs_Diskio *) diskio->adi;
 		coordi = (Abs_Variable *) diskio->coordi;
+		if (adi != NULL)
+			adi->Flush(adi);
 #ifndef HOLD_METADATA
 		{
 			struct timezone tz;
@@ -764,6 +940,7 @@ static char * Make_System_Info_String(sysinf)
 			free(coordi);
 		}
 		if(adi != NULL) {
+			adi->DestructChildInterfaces(adi);
 			adi->Destruct(adi);
 			free(adi);
 		}
@@ -945,7 +1122,17 @@ static int ffCreateVarAndForwardMsg(diskio, msg)
 	if ((varchild = Create("variable", varname, diskio, &newaction, varindex)) == NULL)
 		return 0; 
 	
-	/* Forward the message to the child variable */
+	if (strcmp(diskio->fileformat, "FMT1") == 0) {
+		Abs_Diskio *adi = (Abs_Diskio *)diskio->adi;
+		Abs_Variable *avi = (Abs_Variable *)SelfVariable_Interface(varchild);
+		
+		adi->SetVariableInterface(adi, varindex, avi);
+	/* If FMT1, make the fields of the element ro and hidden */
+		return ffProtectVariableFields(varchild);
+	}
+
+	/* Forward the message to the child variable if the 
+	   underlying format supports the notion of named variables */
 	AddForwMsg(varchild, msg->src, msg->type, MSGSLOT(msg), msg->nslots);
 
 	return 1;
@@ -956,6 +1143,7 @@ static int ffCreateVarAndForwardMsg(diskio, msg)
 static int ffSetUpInterpols(diskio)
  register struct Gen_Diskio_Type *diskio;
 {
+        char *SelfMetadata_Interface();
 	int framesize;
 	Element *child;
 	Abs_Metadata **SAVEamdi;
@@ -1009,12 +1197,87 @@ static int ffSetUpInterpols(diskio)
   	return framesize;
 }
 
+static int ffCreateDefaultGlobalMetadata(diskio) 
+ register struct Gen_Diskio_Type *diskio;	
+{
+	Action newaction;
+	char timebuf[BUFSIZ];
+	char *newargv[20];
+	int j;
+	Element *child;
+	int headerexists = 0;
+
+	for (child = diskio->child; child != NULL; child=child->next) {
+		if((strcmp(BaseObject(child)->name, "metadata") == 0) &&
+			(strcmp(child->name, "start_time") == 0))
+			headerexists = 1;
+	}
+
+	if (!headerexists) {
+		newaction.name = "CREATE";
+		newaction.type = CREATE;
+		newaction.data = (char *) diskio;
+	/*
+	Create default global metadata children one-by-one with 
+	the appropriate default arguments 	
+	*/
+
+	/** FIRST - ############################################## */
+
+	/** The 'start_time' metadata element **/
+	j = 0;	
+	newargv[j] = "metadata"; j++;
+	newargv[j] = "start_time"; j++;
+
+	newargv[j] = "-values"; j++;
+	sprintf(timebuf, "%f", SimulationTime());
+	newargv[j] = timebuf; j++;
+
+	newargv[j] = "-size"; j++;
+	newargv[j] = "1"; j++;
+
+	newargv[j] = "-type"; j++;
+	newargv[j] = "float32"; j++;
+
+	newaction.argc = j;
+	newaction.argv = newargv;
+	if (Create("metadata", "start_time", diskio, &newaction, 0) == NULL)
+		return 0;
+
+		/** ############################################## */
+
+		/** SECOND - ############################################## */
+
+		/** The 'dt' metadata element **/
+		j = 0;	
+		newargv[j] = "metadata"; j++;
+		newargv[j] = "dt"; j++;
+
+		newargv[j] = "-values"; j++;
+		sprintf(timebuf, "%f", Clockrate(diskio));
+		newargv[j] = timebuf; j++;
+
+		newargv[j] = "-size"; j++;
+		newargv[j] = "1"; j++;
+
+		newargv[j] = "-type"; j++;
+		newargv[j] = "float32"; j++;
+
+		newaction.argc = j;
+		if (Create("metadata", "dt", diskio, &newaction, 0) == NULL)
+			return 0;
+	}
+	
+	return 1;
+}
+
 static int ffCreateMetadataAndForwardMsg(diskio, msg)
  register struct Gen_Diskio_Type *diskio;
  MsgIn *msg;
 {
 #define MAX_CREATEARGS 20
 
+        char *SelfMetadata_Interface();
 	char *newargv[50];
 	Action newaction;
 	char *targv[MAX_CREATEARGS];
@@ -1064,8 +1327,19 @@ static int ffCreateMetadataAndForwardMsg(diskio, msg)
 	/* Create the child variable */
 	if ((mdchild = Create("metadata", mdname, diskio, &newaction, mdindex)) == NULL)
 		return 0; 
+
+	if (strcmp(diskio->fileformat, "FMT1") == 0) {
+		Abs_Diskio * adi = (Abs_Diskio *) diskio->adi;		
+		Abs_Metadata *amdi = (Abs_Metadata *)SelfMetadata_Interface(mdchild);
+
+		adi->SetMetadataInterface(adi, mdindex, amdi);
+	/* If FMT1, make the fields of the element ro */
+		return ffProtectMetadataFields(mdchild);
+	}
 	
-	/* Forward the message to the child variable */
+	/* 
+	Forward the message to the child metadata element
+	*/
 	AddForwMsg(mdchild, msg->src, msg->type, MSGSLOT(msg), msg->nslots);
 
 	return 1;
@@ -1073,6 +1347,7 @@ static int ffCreateMetadataAndForwardMsg(diskio, msg)
 #undef MAX_CREATEARGS 
 }
 
+
 static int ffDetermineNewIndex(diskio, childname)
  register struct Gen_Diskio_Type *diskio;
  char *childname;
@@ -1103,6 +1378,7 @@ static int ffDetermineNewIndex(diskio, childname)
 static void ffRecreateHierarchy(diskio)
  register struct Gen_Diskio_Type *diskio;
 {
+        char *SelfMetadata_Interface();
   	char *newargv[50];
   	Action newaction; 
 	int ngmd, nvar;
@@ -1133,12 +1409,19 @@ static void ffRecreateHierarchy(diskio)
 	  */
 		ngmd = adi->nglobmetadata;
 		for(i=0; i<ngmd; i++){
+			Element *mdchild;
 			mdname = adi->RecreateNameAndIndex(adi, adi->globmetadatanames[i], &mdindex);	
 			if(strcmp(mdname, "autoinactiveSAVE_diskio")==0) {
 				FreeString(mdname);
 				continue;
 			}
-			Create("metadata", mdname, diskio, &newaction, mdindex);	
+			mdchild = Create("metadata", mdname, diskio, &newaction, mdindex);	
+			if (strcmp(diskio->fileformat, "FMT1") == 0) {
+				Abs_Metadata *amdi;
+				amdi = (Abs_Metadata *) SelfMetadata_Interface(mdchild);	
+				adi->SetMetadataInterface(adi, mdindex, amdi);
+				ffProtectMetadataFields(mdchild);
+			}
 			FreeString(mdname);
 		}
 	}
@@ -1150,16 +1433,24 @@ static void ffRecreateHierarchy(diskio)
 	if(adi->nvariables > 0){
 		char *varname;
 		int varindex;
+		Abs_Variable *avi;
 		nvar = adi->nvariables;
 		for(i=0; i<nvar; i++){
+			Element *varchild;
 			varname = adi->RecreateNameAndIndex(adi, adi->variablenames[i], &varindex);	
 			if (strcmp(varname, "autoinactiveSAVE_diskio") == 0) {
 				FreeString(varname);
 				continue;
 			}
-			Create("variable", varname, diskio, &newaction, varindex);	
+			varchild = Create("variable", varname, diskio, &newaction, varindex);	
+			if (strcmp(diskio->fileformat, "FMT1") == 0) {
+				avi = (Abs_Variable *) SelfVariable_Interface(varchild);	
+				adi->SetVariableInterface(adi, varindex, avi);
+				ffProtectVariableFields(varchild);
+			}
 			FreeString(varname);
 		}
+		adi->FillAllVariableValues(adi);
   	}
 }
 
