@@ -1,16 +1,41 @@
 /*********************************************************************
 
-** This library uses the netcdf - version 2.4.3 which is
+** This library uses the netcdf - version 3.4 which is
 ** Copyright 1993-1997 University Corporation for Atmospheric Research/Unidata
 
 ** The netcdf library is provided as per the terms of the
-** UCAR/Unidata license
+** UCAR/Unidata license.
 
-** See netcdf-2.4.3/COPYRIGHT for the full notice
+** See netcdf-3.4/copyright.html for the full notice.
 
 ********************************************************************/
-/* $Id: absff_variable.c,v 1.2 1997/07/26 00:15:24 venkat Exp $ */
-/* $Log: absff_variable.c,v $
+/* $Id: absff_variable.c,v 1.7 2000/06/12 04:38:21 mhucka Exp $ */
+/*
+ * $Log: absff_variable.c,v $
+ * Revision 1.7  2000/06/12 04:38:21  mhucka
+ * 1) Removed nested comments in RCS/CVS log lines, to quiet down the
+ *    IRIX cc compiler.
+ * 2) Added NOTREACHED comments where appropriate.
+ *
+ * Revision 1.6  1999/10/16 21:53:59  mhucka
+ * Merged Upi's changes for supporting netCDF 3.4.  The changes in most files
+ * only involve the copyright notice.  I also fixed up the comments slightly
+ * to reflect the location of the actual netCDF copyright file.
+ *
+ * Revision 1.5  1999/10/16 21:25:06  mhucka
+ * Venkat had a version of the diskio code in his home directory that he
+ * appears to have been working on, and that did not make it into the
+ * source tree that I had originally used as the starting point of my
+ * GENESIS 2.2.2 DR reorganization effort.  In order to merge Venkat's
+ * changes into the distribution, I'm reverting to his version, then will
+ * update his version to incorporate recent changes by Upi for netCDF 3.4.
+ *
+ * Revision 1.3  1998/01/14 18:16:42  venkat
+ * Replaced usage of no longer defined typdefs (uchar, ushort etc.,) with
+ * the appropriate type names. Also replaced old variable-specific
+ * function calls with the new generalized equivalents (Absff_CopyToValues(),
+ * Absff_AllocDefaultValues() etc.) defined in absff_utilfunc.c.
+ *
  * Revision 1.2  1997/07/26 00:15:24  venkat
  * Added-Copyright-notice-and-RCS-headers
  * */
@@ -96,13 +121,13 @@ int Absff_SetVariableValuesFromString(avi, vals, start_index, count)
  Abs_Varshape *shape;
  void *values;
 
- uchar *uchar_values;
+ unsigned char *uchar_values;
  char *char_values;
- ushort *ushort_values;
+ unsigned short *ushort_values;
  short *short_values;
- uint *uint_values;
+ unsigned int *uint_values;
  int *int_values;
- ulong *ulong_values;
+ unsigned long *ulong_values;
  long *long_values;
  float *float_values;
  double *double_values;
@@ -144,10 +169,10 @@ int Absff_SetVariableValuesFromString(avi, vals, start_index, count)
  		for(i=0; i<ndims; i++){
 			oldsize = (i==0)? count[0]:oldsize+count[i];
 			if(values == NULL) 
-				values = (uchar *) malloc((int)count[0] * sizeof(unsigned char));
+				values = (unsigned char *) malloc((int)count[0] * sizeof(unsigned char));
 			else
-				values = (uchar *) realloc(values, oldsize*sizeof(unsigned char));	
-			uchar_values = (uchar *)values;
+				values = (unsigned char *) realloc(values, oldsize*sizeof(unsigned char));	
+			uchar_values = (unsigned char *)values;
 			for(j=start; j<count[i]; j++)
 				uchar_values[j] = vals[j][0];		
 			start+=(int)count[i];
@@ -158,10 +183,10 @@ int Absff_SetVariableValuesFromString(avi, vals, start_index, count)
  		for(i=0; i<ndims; i++){
 			oldsize = (i==0)? count[0]:oldsize+count[i];
 			if(values == NULL) 
-				values = (ushort *) malloc((int)count[0] * sizeof(unsigned short));
+				values = (unsigned short *) malloc((int)count[0] * sizeof(unsigned short));
 			else
-				values = (ushort *) realloc(values, oldsize*sizeof(unsigned short));	
-			ushort_values = (ushort *)values;
+				values = (unsigned short *) realloc(values, oldsize*sizeof(unsigned short));	
+			ushort_values = (unsigned short *)values;
 			for(j=start; j<count[i]; j++)
 				ushort_values[j] = atoi(vals[j]);		
 			start+=(int)count[i];
@@ -185,10 +210,10 @@ int Absff_SetVariableValuesFromString(avi, vals, start_index, count)
 		for(i=0; i<ndims; i++){
 			oldsize = (i==0)? count[0]:oldsize+count[i];
                         if(values == NULL)
-                                values = (uint *) malloc((int)count[0] * sizeof(unsigned int));
+                                values = (unsigned int *) malloc((int)count[0] * sizeof(unsigned int));
                         else
-                                values = (uint *) realloc(values, oldsize * sizeof(unsigned int));
-			uint_values = (uint *) values;
+                                values = (unsigned int *) realloc(values, oldsize * sizeof(unsigned int));
+			uint_values = (unsigned int *) values;
                         for(j=start; j<count[i]; j++)
                                 uint_values[j] = atoi(vals[j]);
                         start+=(int)count[i];
@@ -231,7 +256,7 @@ int Absff_SetVariableValuesFromString(avi, vals, start_index, count)
                                 values = (unsigned long *) malloc((int)count[0] * sizeof(unsigned long));
                         else
                                 values = (unsigned long *) realloc(values, oldsize * sizeof(unsigned long));
-			ulong_values = (ulong *) values;
+			ulong_values = (unsigned long *) values;
                         for(j=start; j<count[i]; j++)
                                 ulong_values[j] = Absff_Atol(vals[j]);
                         start+=(int)count[i];
@@ -271,11 +296,13 @@ int Absff_SetVariableValuesFromString(avi, vals, start_index, count)
 	case abs_string:
                 /* No conversion necessary */
                 return avi->SetValues(avi, vals, start_index, count);
+		/* NOTREACHED */
                 break;
 
  	default:
 		/* Error: Invalid type */
 		return -1;
+		/* NOTREACHED */
 		break;
 	}
 	
@@ -308,10 +335,12 @@ size_t Absff_GetVariableTypeSize(avi)
   switch (avi->abstype) {
 	case abs_float32:
 		return ((size_t) sizeof(float));
+		/* NOTREACHED */
 		break;
 
 	case abs_float64:
 		return ((size_t) sizeof(double));
+		/* NOTREACHED */
 		break;
 
 	/* Need to fill in for other types as well */
@@ -337,13 +366,13 @@ char buffer[50];
 void *localvals;
 int numvals;
 
- uchar *uchar_values;
+ unsigned char *uchar_values;
  char *char_values;
- ushort *ushort_values;
+ unsigned short *ushort_values;
  short *short_values;
- uint *uint_values;
+ unsigned int *uint_values;
  int *int_values;
- ulong *ulong_values;
+ unsigned long *ulong_values;
  long *long_values;
  float *float_values;
  double *double_values;
@@ -365,8 +394,8 @@ int numvals;
  switch(type){
 	case abs_uchar8:
 	case abs_uint8:
-		localvals = (uchar *) malloc(numvals * sizeof(uchar));
-		uchar_values = (uchar *) localvals;
+		localvals = (unsigned char *) malloc(numvals * sizeof(unsigned char));
+		uchar_values = (unsigned char *) localvals;
 		avi->GetValues(avi, localvals, start_index, count);
 		if(localvals != NULL) {
 			for(i=0; i<ndims; i++){
@@ -398,8 +427,8 @@ int numvals;
 		break;
 		
 	case abs_uint16:
-		localvals = (ushort *) malloc(numvals * sizeof(unsigned short));
-		ushort_values = (ushort *) localvals;
+		localvals = (unsigned short *) malloc(numvals * sizeof(unsigned short));
+		ushort_values = (unsigned short *) localvals;
 		avi->GetValues(avi, localvals, start_index, count);
 		if(localvals != NULL) {
 			for(i=0; i<ndims; i++){
@@ -430,8 +459,8 @@ int numvals;
 		break;
 	
  	case abs_uint32:
-		localvals = (uint *) malloc(numvals * sizeof(unsigned int));
-		uint_values = (uint *)localvals;
+		localvals = (unsigned int *) malloc(numvals * sizeof(unsigned int));
+		uint_values = (unsigned int *)localvals;
 		avi->GetValues(avi, localvals, start_index, count);
 		if(localvals != NULL) {
 			for(i=0; i<ndims; i++){
@@ -461,8 +490,8 @@ int numvals;
 		break;
 
 	case abs_uint64:
-		localvals = (ulong *) malloc(numvals * sizeof(unsigned long));
-		ulong_values = (ulong *) localvals;
+		localvals = (unsigned long *) malloc(numvals * sizeof(unsigned long));
+		ulong_values = (unsigned long *) localvals;
 		avi->GetValues(avi, localvals, start_index, count);
 		if(localvals != NULL) {
 			for(i=0; i<ndims; i++){
@@ -601,7 +630,7 @@ shape = avi->varshape;
  else
 	varsize = 0;
 
- Absff_ConvertVariableValueFromString(avi, input, &passed_input);
+ Absff_ConvertVariableValueFromString(avi, input, (void *) &passed_input);
  
  ret_index = 0;
  count[0] = 1;
@@ -609,7 +638,7 @@ shape = avi->varshape;
  for(i= 0; i< varsize; i++){
 	start_index[0] = i;	
 	avi->GetValuesAsString(avi, vals, start_index, count);
-	Absff_ConvertVariableValueFromString(avi, vals[0], &actual_input);
+	Absff_ConvertVariableValueFromString(avi, vals[0], (void *) &actual_input);
 	Absff_FreeString(vals[0]);
 	ret_index = i;
 	if(!(comparestat=Absff_ActualVariableValueIsSmaller(avi, actual_input, passed_input))) 
@@ -657,7 +686,7 @@ shape = avi->varshape;
  else
 	varsize = 0;
 
- Absff_ConvertVariableValueFromString(avi, input, &passed_input);
+ Absff_ConvertVariableValueFromString(avi, input, (void *) &passed_input);
  
  count[0] = 1;
 
@@ -672,7 +701,7 @@ shape = avi->varshape;
  for(i= avi->last_retrieved_index; i< varsize; i++){
 	start_index[0] = i;	
 	avi->GetValuesAsString(avi, vals, start_index, count);
-	Absff_ConvertVariableValueFromString(avi, vals[0], &actual_input);
+	Absff_ConvertVariableValueFromString(avi, vals[0], (void *) &actual_input);
 	Absff_FreeString(vals[0]);
 	avi->last_retrieved_index = i;
 	if(!(comparestat=Absff_ActualVariableValueIsSmaller(avi, actual_input, passed_input))) 
