@@ -1,12 +1,22 @@
 /*
-** $Id: hh_struct.h,v 1.1 1992/12/11 19:03:03 dhb Exp $
+** $Id: hh_struct.h,v 1.3 1998/06/30 21:04:58 dhb Exp $
 ** $Log: hh_struct.h,v $
+** Revision 1.3  1998/06/30 21:04:58  dhb
+** Moved define for HH_CHAN_TYPE to hh_struct_defs.h and in clude that
+** here.  Avoids multiple includes of sim headers when including hh_struct.h
+** in olf.
+**
+** Revision 1.2  1998/04/29 12:55:05  dhb
+** Pulled HH specific parameter fields out into a define so they
+** could be used in the tabchannel version of hh_channel.
+**
 ** Revision 1.1  1992/12/11 19:03:03  dhb
 ** Initial revision
 **
 */
 
 #include "struct_defs.h"
+#include "hh_struct_defs.h"
 
 struct vdep_gate_type {
     SEGMENT_TYPE
@@ -46,21 +56,6 @@ struct hh_channel_type {
     double 	Y;
     float   	Xpower;
     float	Ypower;
-    short   	X_alpha_FORM;
-    float   	X_alpha_A;
-    float   	X_alpha_B;
-    float   	X_alpha_V0;
-    short   	X_beta_FORM;
-    float   	X_beta_A;
-    float   	X_beta_B;
-    float   	X_beta_V0;
-    short   	Y_alpha_FORM;
-    float   	Y_alpha_A;
-    float   	Y_alpha_B;
-    float   	Y_alpha_V0;
-    short   	Y_beta_FORM;
-    float   	Y_beta_A;
-    float   	Y_beta_B;
-    float   	Y_beta_V0;
+    HH_CHAN_TYPE
 };
 
