@@ -1,12 +1,23 @@
-static char rcsid[] = "$Id: compt_update.c,v 1.3 1997/05/28 22:27:14 dhb Exp $";
+static char rcsid[] = "$Id: compt_update.c,v 1.5 1999/10/17 21:39:36 mhucka Exp $";
 
-/*  Version EDS20i 95/06/08, Erik De Schutter, December 1991-March 1994
+/*  Version EDS22c 98/03/07, Erik De Schutter, December 1991-March 1998
 ** Upinder S. Bhalla Caltech May-December 1991 */
 
 #include "hines_ext.h"
 
 /*
 ** $Log: compt_update.c,v $
+** Revision 1.5  1999/10/17 21:39:36  mhucka
+** New version of Hines code from Erik De Schutter, dated 30 March 1999
+** and including an update to hines.c from Erik from June 1999.  This version
+** does not include changes to support Red Hat 6.0 and the Cray T3E, which
+** will need to be incorporated in a separate revision update.
+**
+ * EDS22c revison: EDS BBF-UIA 97/11/28-97/12/20
+ * Made symmetric compartments work
+ * Deleted use of values array, folded diagonals into results array like
+ *  for chnamode>1
+ *
 ** Revision 1.3  1997/05/28 22:27:14  dhb
 ** Update from Antwerp GENESIS 21e
 **
@@ -19,6 +30,7 @@ static char rcsid[] = "$Id: compt_update.c,v 1.3 1997/05/28 22:27:14 dhb Exp $";
 
 
 /* Simple function for putting data values back into elements */
+/* Chanmode 0-1 only */
 do_compt_update(hsolve)
 	Hsolve	*hsolve;
 {
@@ -36,11 +48,11 @@ do_compt_update(hsolve)
 	if (BaseObject(hsolve)->method == CRANK_INT) {
 		for(i=0;i<ncompts;i++) {
 			v = &(compts[elmnum[i]]->Vm);
-			*v = 2.0*results[i]-(*v);
+			*v = 2.0*results[2*i]-(*v);
 		}
 	} else { /* BEULER by default */
 		for(i=0;i<ncompts;i++) {
-			compts[elmnum[i]]->Vm = results[i];
+			compts[elmnum[i]]->Vm = results[2*i];
 		}
 	}
 }
