@@ -1,9 +1,62 @@
-static char rcsid[] = "$Id: hines_chip_init.c,v 1.1 1997/05/28 23:11:50 dhb Exp $";
+static char rcsid[] = "$Id: hines_chip_init.c,v 1.13 2001/06/29 21:14:56 mhucka Exp $";
 
-/* Version EDS21e 97/05/05, Erik De Schutter, Caltech & BBF-UIA 4/92-5/97 */
+/* Version EDS22h* 99/09/13, Erik De Schutter, Caltech & BBF-UIA 4/92-9/99 */
 
 /*
 ** $Log: hines_chip_init.c,v $
+** Revision 1.13  2001/06/29 21:14:56  mhucka
+** Removed a nested comment to quiet the compiler.
+**
+** Revision 1.12  2001/04/18 22:39:36  mhucka
+** Miscellaneous portability fixes, mainly for SGI IRIX.
+**
+** Revision 1.10  2001/01/01 01:19:06  mhucka
+** Fix from Hugo Cornelis for a bug in hsolve in which Vm is loaded too late
+** under sone circumstances.
+**
+** Revision 1.9  2000/09/21 19:31:27  mhucka
+** One of the printfs in h_hh_chip_init() didn't cast an argument properly.
+**
+** Revision 1.8  2000/09/11 15:49:10  mhucka
+** I botched the merger of Hugo's changes on one line.  Now fixed.
+**
+** Revision 1.7  2000/09/07 02:22:55  mhucka
+** Fixes from Hugo Cornelis for various initialization problems.
+**
+** Revision 1.6  1999/10/18 00:10:31  mhucka
+** Additional fixes from Erik De Schutter, for a bug in calculating dt.
+**
+** Revision 1.5  1999/10/17 23:15:40  mhucka
+** Attempted to merge in changes from Greg Hood for fixing problems on
+** the Cray T3E.  The same changes also make it possible to compile with
+** optimization turned on under Red Hat 6.0's default version of gcc/egcs.
+**
+** Revision 1.4  1999/10/17 21:39:36  mhucka
+** New version of Hines code from Erik De Schutter, dated 30 March 1999
+** and including an update to hines.c from Erik from June 1999.  This version
+** does not include changes to support Red Hat 6.0 and the Cray T3E, which
+** will need to be incorporated in a separate revision update.
+**
+** EDS22k revision HC BBF-UIA 00/07/24
+** no normalization for incoming ACTIVATION msg's on hsolve'd synchans
+** Fix for case when Vm was loaded to late (for Ca channel only with nernst Ek)
+**
+** EDS22h revison: EDS BBF-UIA 99/07/14-99/09/13
+** Corrected error in in tables when dt was changed 
+**
+** EDS22f revison: EDS BBF-UIA 98/10/28-98/11/23
+** Minor bug corrections: Mg_block, taupump
+**
+** EDS22d revison: EDS BBF-UIA 98/05/04-98/09/30
+** Added support for single shells and instant gates
+**
+** EDS22c revison: EDS BBF-UIA 97/12/04-97/12/05
+** Deleted use of values array
+** Correct SYNCHAN time normalization
+**
+** EDS22b revison: EDS BBF-UIA 97/11/17-97/11/25
+** Added concpool and hillpump code, reorganized some arrays
+**
 ** Revision 1.1  1997/05/28 23:11:50  dhb
 ** Initial revision
 **
@@ -22,206 +75,6 @@ static char rcsid[] = "$Id: hines_chip_init.c,v 1.1 1997/05/28 23:11:50 dhb Exp
 
 #include "hines_ext.h"
 
-/* This routine sets up the func array for fast solution of the 
-** sparse matrix. It does this in two passes : first, it finds
-** the number of functions, second it fills them up.  */
-/* chanmode 2/3 version: diag elements in results array */
-/* A new ravals array is allocated which copies the values array in 
-**  order of funcs access.  This is a bit of waste of space as values
-**  is no longer used, but setting up ravals otherwise is non-trivial */
-/* Now also sets up communication of results array in parallel mode */
-int do_chip_hsetup(hsolve)
-	Hsolve	*hsolve;
-{
-	int	i,j,k,n;
-	int	row;
-	int	ind1,ind2,ind3;
-	int	ncompts;
-	int	numnodes,nextborder,nextnode;
-	int	*ri,*cip,*diag,*startnodes;
-	int	nextcip,di,cipnextrow;
-	int	cipi,drow;
-	int	justcount,nocomm;
-	int	nfuncs=0,nravals=0;
-	int	*funcs;
-	double	*ravals,*values,*chip;
-
-	ncompts=hsolve->ncompts;
-	values=hsolve->values;
-	chip = hsolve->chip;
-	ri=hsolve->ri;
-	cip=hsolve->cip;
-	diag=hsolve->diag;
-	numnodes = hsolve->numnodes;
-	if ((numnodes>1) && (hsolve->readflag==HREAD_T)) {
-	    startnodes=hsolve->startnodes;
-	    nextborder=startnodes[1];
-	    nextnode=2;
-	} else {
-	    nextborder=ncompts;
-	}
-
-	if (hsolve->nfuncs > 0) {
-	    justcount = 0;
-	    if (hsolve->funcs) {
-		funcs=hsolve->funcs;
-		ravals=hsolve->ravals;
-	    } else {
-		hsolve->funcs=funcs=(int *)calloc(hsolve->nfuncs,sizeof(int));
-		hsolve->ravals=ravals=(double *)calloc(hsolve->nravals,sizeof(double));
-	    }
-	} else {
-	    justcount = 1;
-	}
-
-	/* looping over all rows, doing forward substitution */
-	for (i=0;i<ncompts;i++) {
-	    di=diag[i];
-/*		resultvalue=results[2*i];		*/
-/*		diavalue = results[2*i+1];		*/
-	    if (i>=nextborder) {
-		nocomm=1;
-		if (nextnode<numnodes) {
-		    nextborder=startnodes[nextnode];
-		    nextnode++;
-		} else {
-		    nextborder=ncompts;
-		}
-	    } else {
-		nocomm=0;
-	    }
-	    if (justcount) {
-		if (nocomm) nfuncs++;
-		if (i>0) nfuncs+=1;
-	    } else {
-		if (nocomm) funcs[nfuncs++]=RECRESULT;
-		if (i>0) funcs[nfuncs++]=SET_DIAG;
-	    }
-	    di++;
-	    nextcip=cip[i+1];
-	    /* Looping over all coupled rows */
-	    for (j=di;j<nextcip;j++) {
-		row=ri[j]; /* since the matrix is symmetrical, the ri also
-			   ** gives the correct row index */
-		cipnextrow=cip[row+1];
-		for (ind1=cip[row];ind1<cipnextrow;ind1++) {
-		    if (ri[ind1]==i) {
-			/* calculate scaling factor */
-/*				temp = values[ind1]/diavalue;			*/
-/*				results[row] -= resultvalue*temp;		*/
-			if (row>=nextborder) {
-			    nocomm=1;
-			    if (nextnode<numnodes) {
-				nextborder=startnodes[nextnode];
-				nextnode++;
-			    } else {
-				nextborder=ncompts;
-			    }
-			} else {
-			    nocomm=0;
-			}
-			if(justcount){
-			    if (nocomm) nfuncs++;
-			    nfuncs+=2;
-			    nravals++;
-			} else {
-			    if (nocomm) funcs[nfuncs++]=RECRESULT;
-			    funcs[nfuncs++]=SCALE;
-			    funcs[nfuncs++]=2*row;
-			    ravals[nravals++]=values[ind1];
-			}
-			/* looping over all nonzero columns for _ith_ row */
-			for (ind2=di,ind3=ind1+1;ind2<nextcip;ind2++){
-			    for (;ri[ind3]!=ri[ind2];ind3++) {
-				if(ind3>=cipnextrow) {
-				    /* a major screw-up */
-				    Error();
-				    printf(" during SETUP of %s: bug in forward elim.\n",Pathname(hsolve));
-				    return(ERR);
-				}
-			    }
-			    /* Otherwise, proceed with elimination */
-/*					values[ind3] -= values[ind2]*temp;
-*/
-			    if (ri[ind3]>=nextborder) {
-				nocomm=1;
-				if (nextnode<numnodes) {
-				    nextborder=startnodes[nextnode];
-				    nextnode++;
-				} else {
-				    nextborder=ncompts;
-				}
-			    } else {
-				nocomm=0;
-			    }
-			    if (justcount) {
-				if (nocomm) nfuncs++;
-				nfuncs+=2;
-				nravals++;
-			    } else {
-				if (nocomm) funcs[nfuncs++]=RECRESULT;
-				funcs[nfuncs++]=ARRAY_ELIM;
-				funcs[nfuncs++]=2*ri[ind3]+1;	/* always a diag term */
-				ravals[nravals++]=values[ind2];
-			    }
-			}
-		    }
-		}
-	    }
-	}
-	/* looping over all rows, doing backwards elimination */
-	if ((numnodes>1) && (hsolve->readflag==HREAD_T)) {
-	    nextborder=startnodes[numnodes-1];
-	    nextnode=numnodes-2;
-	} else {
-	    nextborder=0;
-	}
-	for(i=ncompts-1;i>=0;i--) {
-	    di=diag[i];
-	    cipi=cip[i];
-/*		results[i] = temp = results[i]/values[di];		*/
-	    if (i<nextborder) {
-		nocomm=1;
-		nextborder=startnodes[nextnode];
-		nextnode--;
-	    } else {
-		nocomm=0;
-	    }
-	    if(justcount){
-		if (nocomm) nfuncs++;
-		nfuncs+=1;
-	    } else {
-		if (nocomm) funcs[nfuncs++]=SENDVM;
-		funcs[nfuncs++]=CALC_RESULTS;
-	    }
-	    for(j=di-1;j>=cipi;j--) {
-		/* since the matrix is symmetrical, the ri also
-				  ** gives the correct row index */
-		drow=diag[row=ri[j]];
-		for (ind1=cip[row+1]-1;ind1>drow;ind1--) {
-		    if (ri[ind1]==i) {
-/*					results[row] -= values[ind1]*temp;	*/
-			if (justcount) {
-			    nfuncs+=2;
-			    nravals++;
-			} else {
-			    funcs[nfuncs++]=ARRAY_ELIM;
-			    funcs[nfuncs++]=2*row;
-			    ravals[nravals++]=values[ind1];
-			}
-		    }
-		}
-	    }
-	}
-	if (justcount) {
-	    nfuncs++;
-	    hsolve->nfuncs=nfuncs;
-	    hsolve->nravals=nravals;
-	} else {
-	    funcs[nfuncs++]=FINISH;
-	}
-	return(0);
-}
 /* Set of routines to make composite table for tab(2D)channels */
 /* In hsolve, voltage dependent tables are always indexed as [x], concentration
 **  dependent as [y], 2D tables as [x,y] */
@@ -252,9 +105,9 @@ double *tab2Dptr(ip2)
 }
 
 int check_tab(ip,hsolve,must,type,index_type)
-	Interpol	*ip;
+	Interpol *ip;
 	Hsolve 	*hsolve;
-	int		must,type,index_type;
+	int	must,type,index_type;
 {
 	double	*table;
 	Interpol2D *ip2;
@@ -399,7 +252,7 @@ int check_tab(ip,hsolve,must,type,index_type)
 	tablist[ntable]=table;
 	tabcols[ntable]=j;
 	if (must<0) {	/* TABCURR_T */
-	    tabintegrate[ntable]=0;
+	    tabintegrate[ntable]=must;
 	} else {		/* TABCHAN_T or TAB2CHAN_T */
 	    tabintegrate[ntable]=1;
 	}
@@ -410,24 +263,29 @@ int check_tab(ip,hsolve,must,type,index_type)
 int check_chanX(chan,hsolve,type)
 	Element	*chan;
 	Hsolve	*hsolve;
-	int		type;
+	int	type;
 {
-	int	ia,ib;
+	int	ia,ib,must;
 	Tchan	*tchan;
 	T2chan	*t2chan;
 
 	tchan=(Tchan *)chan;
+	if (tchan->instant & INSTANTX) {
+	    must=-1;	/* instant gate */
+	} else {
+	    must=0;	/* normal integration */
+	}
 	if ((tchan->Xpower<1) || (tchan->Xpower>6)) {
 	    Error();
 	    printf(" during SETUP of %s: Xpower of %s should be in range 1-6 for chanmode >= 2.\n",Pathname(hsolve),Pathname(chan));
 	    return(ERR);
 	}
 	if (type==TABCHAN_T) {
-	    ib=check_tab(tchan->X_B,hsolve,0,1,VOLT_INDEX);
+	    ib=check_tab(tchan->X_B,hsolve,must,1,VOLT_INDEX);
 	    if (ib!=ERR) ia=check_tab(tchan->X_A,hsolve,ib+1,1,VOLT_INDEX);
 	} else {
 	    t2chan=(T2chan *)chan;
-	    ib=check_tab(t2chan->X_B,hsolve,0,2,t2chan->Xindex);
+	    ib=check_tab(t2chan->X_B,hsolve,must,2,t2chan->Xindex);
 	    if (ib!=ERR) ia=check_tab(t2chan->X_A,hsolve,ib+1,2,t2chan->Xindex);
 	}
 	if ((ib==ERR) || (ia==ERR)) {
@@ -441,24 +299,29 @@ int check_chanX(chan,hsolve,type)
 int check_chanY(chan,hsolve,type)
 	Element	*chan;
 	Hsolve	*hsolve;
-	int		type;
+	int	type;
 {
-	int	ia,ib;
+	int	ia,ib,must;
 	Tchan	*tchan;
 	T2chan	*t2chan;
 
 	tchan=(Tchan *)chan;
+	if (tchan->instant & INSTANTY) {
+	    must=-1;	/* instant gate */
+	} else {
+	    must=0;	/* normal integration */
+	}
 	if ((tchan->Ypower<1) || (tchan->Ypower>6)) {
 	    Error();
 	    printf(" during SETUP of %s: Ypower of %s should be in range 1-6 for chanmode >= 2.\n",Pathname(hsolve),Pathname(chan));
 	    return(ERR);
 	}
 	if (type==TABCHAN_T) {
-	    ib=check_tab(tchan->Y_B,hsolve,0,1,VOLT_INDEX);
+	    ib=check_tab(tchan->Y_B,hsolve,must,1,VOLT_INDEX);
 	    if (ib!=ERR) ia=check_tab(tchan->Y_A,hsolve,ib+1,1,VOLT_INDEX);
 	} else {
 	    t2chan=(T2chan *)chan;
-	    ib=check_tab(t2chan->Y_B,hsolve,0,2,t2chan->Yindex);
+	    ib=check_tab(t2chan->Y_B,hsolve,must,2,t2chan->Yindex);
 	    if (ib!=ERR) ia=check_tab(t2chan->Y_A,hsolve,ib+1,2,t2chan->Yindex);
 	}
 	if ((ib==ERR) || (ia==ERR)) {
@@ -472,24 +335,29 @@ int check_chanY(chan,hsolve,type)
 int check_chanZ(chan,hsolve,type)
 	Element	*chan;
 	Hsolve	*hsolve;
-	int		type;
+	int	type;
 {
-	int	ia,ib;
+	int	ia,ib,must;
 	Tchan	*tchan;
 	T2chan	*t2chan;
 
 	tchan=(Tchan *)chan;
+	if (tchan->instant & INSTANTZ) {
+	    must=-1;	/* instant gate */
+	} else {
+	    must=0;	/* normal integration */
+	}
 	if ((tchan->Zpower<1) || (tchan->Zpower>6)) {
 	    Error();
 	    printf(" during SETUP of %s: Zpower of %s should be in range 1-6 for chanmode >= 2.\n",Pathname(hsolve),Pathname(chan));
 	    return(ERR);
 	}
 	if (type==TABCHAN_T) {
-	    ib=check_tab(tchan->Z_B,hsolve,0,1,C1_INDEX);
+	    ib=check_tab(tchan->Z_B,hsolve,must,1,C1_INDEX);
 	    if (ib!=ERR) ia=check_tab(tchan->Z_A,hsolve,ib+1,1,C1_INDEX);
 	} else {
 	    t2chan=(T2chan *)chan;
-	    ib=check_tab(t2chan->Z_B,hsolve,0,2,t2chan->Zindex);
+	    ib=check_tab(t2chan->Z_B,hsolve,must,2,t2chan->Zindex);
 	    if (ib!=ERR) ia=check_tab(t2chan->Z_A,hsolve,ib+1,2,t2chan->Zindex);
 	}
 	if ((ib==ERR) || (ia==ERR)) {
@@ -500,22 +368,26 @@ int check_chanZ(chan,hsolve,type)
 	return(0);
 }
 
-int check_stab(hsolve,schan)
+int check_stab(hsolve,cindex)
 	Hsolve	*hsolve;
-	S2chan	*schan;
+	int	cindex;
 {
-	int		j;
+	int	j;
+	S2chan	*schan;
 
+	schan=(S2chan *)hsolve->children[cindex];
 	if (!schan)
 	    return(ERR);
 
 	for(j=0;j<nstables*SYNSIZE;j+=SYNSIZE) {
 	    if ((fabs(stablist[j] - schan->xconst2) < VTINY) &&
 		(fabs(stablist[j+1] - schan->yconst1) < VTINY) &&
-		(fabs(stablist[j+2] - schan->yconst2) < VTINY) &&
-		(fabs(stablist[j+3] - schan->xconst1) < VTINY)) {
-		/* it has already been entered */
-		return(j);
+		(fabs(stablist[j+2] - schan->yconst2) < VTINY)) {
+		if ((fabs(stablist[j+3] - schan->xconst1) < VTINY) || 
+		    (fabs(stablist[j+3] - schan->xconst1/hsolve->dt) < VTINY)) {
+		    /* it has already been entered */
+		    return(j);
+		}
 	    }
 	}
 	if (nstables*SYNSIZE == MAXSYN) {
@@ -526,7 +398,12 @@ int check_stab(hsolve,schan)
 	stablist[j] = schan->xconst2;
 	stablist[j+1] = schan->yconst1;
 	stablist[j+2] = schan->yconst2;
-	stablist[j+3] = schan->xconst1;
+	if (hsolve->childtypes[cindex]==SYNCHANC_T) {
+	    /* normalize to step size by dividing by dt */
+	    stablist[j+3] = schan->xconst1 / hsolve->dt;
+	} else {
+	    stablist[j+3] = schan->xconst1;
+	}
 	nstables++;
 	return(j);
 }
@@ -540,6 +417,7 @@ int h_hh_chip_init(hsolve)
 	int 	nchildren=hsolve->nchildren;
 	Element	**compts,*compt,**children,*child;
 	int 	i,j,k,l,m,n;
+	int * piCompFlags = hsolve->piCompFlags;
 	short   *childtypes;
 	int	*childstart,*childcode,*childmsg,*childpos,*childnames,*childlink;
 	int	ct,cm,cl,cp,cc;
@@ -622,7 +500,6 @@ int h_hh_chip_init(hsolve)
 		    case TAB2CHAN_T:
 		    case TABCURR_T:
 		    case TABGHK_T:
-		    case ELMMPUMP_T:
 			if (cp!=0) {		/* CONCEN msg */
 			    k=abs(cp);
 			    k=childmsg[k];
@@ -636,6 +513,7 @@ int h_hh_chip_init(hsolve)
 		    case CHANC2_T:
 		    case CHANC3_T:
 		    case SYNCHANC_T:
+		    case MGBLOCK_T:
 			if (cl!=0) {		/* flux */
 			    k=childmsg[abs(cl)];
 			    if (k!=lastconc) {
@@ -647,9 +525,10 @@ int h_hh_chip_init(hsolve)
 
 		    case NERNST_T:
 		    case GHK_T:
-			if (cl!=lastconc) {
+		    case ELMMPUMP_T:
+			if (cm!=lastconc) {
 			    nop+=2;		/* CONC_VAL_OP and index */
-			    lastconc=cl;
+			    lastconc=cm;
 			}
 			break;
 
@@ -688,15 +567,6 @@ int h_hh_chip_init(hsolve)
 			    }
 			    nop++;		/* CHAN_EK_OP */
 			}
-			if ((cl!=0)&&(ct!=TABGHK_T)) {        /* flux */
-			    if (cl>0) {		/* simple flux */
-				nop++;		/* FLUX_OP */
-			    } else if (cl<0) {	/* scaled flux */
-				nop++;		/* SCALE_FLUX_OP */
-				nchip++;	/* flux_scale */
-			    }
-			}
-			if (storeall)  nop+=2;	/* STORE_ALLC/I_OP and index */
 			if (ct==TABCHAN_T) {
 			    nop++;		/* ADD_CURR_OP */
 			    if (cc<0) nchip++;	/* DOMAINCONC scale factor */
@@ -744,12 +614,16 @@ int h_hh_chip_init(hsolve)
 			    ** C1_C2_INDEX: NEWCONC1_OP,NEWCONC2_OP,conc,IPOL2C_OP,index,power */
 			    if (tcurr->alloced) {
 				nop+=5;
-				if (check_tab(tcurr->G_tab,hsolve,-1,2,tcurr->Gindex)==ERR)  return(ERR);
-				if (check_tab(tcurr->I_tab,hsolve,-1,2,tcurr->Gindex)==ERR)  return(ERR);
+				k=check_tab(tcurr->G_tab,hsolve,-2,2,tcurr->Gindex);
+				if (k==ERR) {
+				    return(ERR);
+				} else {
+				    if (check_tab(tcurr->I_tab,hsolve,k+1,2,tcurr->Gindex)==ERR)  return(ERR);
+				}
 			    }
 			} else { /* CHANC2_T, CHANC3_T and SYNCHANC_T */
 			    schan=(S2chan *)child;
-			    k=check_stab(hsolve,schan);
+			    k=check_stab(hsolve,j);
 			    if (k<0) {
 				return(ERR);
 			    } else {
@@ -766,13 +640,22 @@ int h_hh_chip_init(hsolve)
 			    if (childtypes[j+1]==MGBLOCK_T) {
 				nchip+=3;	/* KMg_A 1/KMg_B and CMg */
 				nop++;		/* MGBLOCK_OP */
-				j++;
+				cl=childlink[j+1]; /* for flux from Mg_block */
 			    }
 			}
+			if ((cl!=0)&&(ct!=TABGHK_T)) {        /* flux */
+			    if (cl>0) {		/* simple flux */
+				nop++;		/* FLUX_OP */
+			    } else if (cl<0) {	/* scaled flux */
+				nop++;		/* SCALE_FLUX_OP */
+				nchip++;	/* flux_scale */
+			    }
+			}
+			if (storeall)  nop+=2;	/* STORE_ALLC/I_OP and index */
 			break;
 
 		    case MGBLOCK_T:
-			printf("Error in %s: Bug #3 in h_hh_chip_init %d.\n",hsolve->name,j);
+			/* done before in CHANC[23]_T and SYNCHANC_T code */
 			break;
 
 		    case NERNST_T:
@@ -826,7 +709,7 @@ int h_hh_chip_init(hsolve)
 	    compchips[i]=nchip;	/* needed in a lot of places */
 	    n=childstart[i];
 	    /* do first passive compartment structure (is also the boundary) */
-	    h_check_msgs(hsolve,compt,COMPT_OP,i,nop,nchip);
+	    h_check_msgs(hsolve,compt,COMPT_OP,COMPT_T,i,nop,nchip);
 	    if (h_has_output(compt)) h_store_out(hsolve,COMPT_T,i,0);
 	    if (i==0) {
 		ops[nop]=FCOMPT_OP;
@@ -838,6 +721,21 @@ int h_hh_chip_init(hsolve)
 	    nchip+=2;
 	    voltindex=-1;    /* none selected */
 	    lastc1index=lastc2index=0;
+
+	    /*- if this compartment has voltage dependent gates */
+
+	    if (piCompFlags && piCompFlags[i])
+	    {
+		/*- operations to compute voltage index */
+
+		ops[nop] = NEWVOLT_OP;
+		nop++;
+
+		/*- remember volt index (is index of comp) */
+
+		voltindex = i;
+	    }
+
 	    for (j=n; j<childstart[i+1]; j++) {
 		child=children[j];
 		ct=childtypes[j];
@@ -873,11 +771,11 @@ int h_hh_chip_init(hsolve)
 		    case NERNST_T:
 		    case GHK_T:
 		    case ELMMPUMP_T:
-			if (cl!=lastconc) {
+			if (cm!=lastconc) {
 			    ops[nop]=CONC_VAL_OP;
-			    ops[nop+1]=cl;
+			    ops[nop+1]=cm;
 			    nop+=2;
-			    lastconc=cl;
+			    lastconc=cm;
 			}
 			/* no break ! */
 
@@ -889,14 +787,14 @@ int h_hh_chip_init(hsolve)
 		/* fill ops array */
 		switch (ct) {
 		    case SPIKEGEN_T:
-			h_check_msgs(hsolve,child,SPIKE_OP,i,nop,nchip);
+			h_check_msgs(hsolve,child,SPIKE_OP,ct,i,nop,nchip);
 			ops[nop]=SPIKE_OP;
 			nop+=3;		/* SPIKE_OP, counter, refractory period */
 			nchip++;	/* voltage threshold */
 			break;
 
 		    case ELMMPUMP_T:
-			h_check_msgs(hsolve,child,MMPUMP_OP,i,nop,nchip);
+			h_check_msgs(hsolve,child,MMPUMP_OP,ct,i,nop,nchip);
 			ops[nop]=ELMMPUMP_OP;
 			nop++;		/* ELMMPUMP_OP */
 			nchip+=3;	/* Kd, dt.vmax.surf.val.F, dt.vmax.surf/vol */
@@ -976,22 +874,22 @@ int h_hh_chip_init(hsolve)
 					break;
 				}
 				/* do voltage first: no checks required */
-				switch (gindex) {
-				    case VOLT_INDEX:
-				    case VOLT_C1_INDEX:
-				    case VOLT_C2_INDEX:
-				    case VOLT_DOMAIN_INDEX:
-					if (voltindex!=i) { 
-					    /* will be done once for all gates*/
-					    ops[nop]=NEWVOLT_OP;
-					    nop++;
-					    voltindex=i;
-					}
-					break;
+/* 				switch (gindex) { */
+/* 				    case VOLT_INDEX: */
+/* 				    case VOLT_C1_INDEX: */
+/* 				    case VOLT_C2_INDEX: */
+/* 				    case VOLT_DOMAIN_INDEX: */
+/* 					if (voltindex!=i) {  */
+/* 					    / * will be done once for all gates* */
+/* 					    ops[nop]=NEWVOLT_OP; */
+/* 					    nop++; */
+/* 					    voltindex=i; */
+/* 					} */
+/* 					break; */
 				    
-				    default:
-					break;
-				}
+/* 				    default: */
+/* 					break; */
+/* 				} */
 				/* do C1 concentration second:checks required */
 				switch (gindex) {
 				    case VOLT_INDEX:
@@ -1064,7 +962,7 @@ int h_hh_chip_init(hsolve)
 			}
 
 			/* determine source of Ek */
-			if (cm==0) {	/* define Ek */
+			if (cm==0) { /* define Ek */
 			    Ek=((Tchan *)child)->Ek;
 			    if (fabs(Ek-lastEk)<VTINY) {
 				ops[nop]=CHAN_OP;
@@ -1102,23 +1000,19 @@ int h_hh_chip_init(hsolve)
 			/* determine table pointer for tabulated channels */
 			if ((ct<=TABCURR_T) || (ct==TABGHK_T)) {
 			    if (ct<TABCURR_T) {
-				h_check_msgs(hsolve,child,CHAN_OP,i,nop,nchip);
+				h_check_msgs(hsolve,child,CHAN_OP,ct,i,nop,nchip);
 			    } else {
-				h_check_msgs(hsolve,child,TABCURR_OP,i,nop,nchip);
+				h_check_msgs(hsolve,child,TABCURR_OP,ct,i,nop,nchip);
 			    }
 			    /* cycle through the X,Y,Z gates */
 			    for (m=0; m<l; m++) {
 				/* determine type of gate */
 				switch (m) {
 				    case 0:   /* X gate */
-					if (ct==TABCURR_T) {	/* TABCURR_T or TABGHK_T */
+					if ((ct==TABCURR_T) || (ct==TABGHK_T)) {
 					    gindex=tcurr->Gindex;
 					    table=tab2Dptr(tcurr->G_tab);
 					    gpower=TABCURR_OP;
-					} else if (ct==TABGHK_T) {	/* TABCURR_T or TABGHK_T */
-					    gindex=tcurr->Gindex;
-					    table=tab2Dptr(tcurr->G_tab);
-					    gpower=TABGHK_OP;
 					} else {
 					    if (!tchan->X_alloced) continue;
 					    if (ct==TAB2CHAN_T) {
@@ -1129,6 +1023,8 @@ int h_hh_chip_init(hsolve)
 						table=tchan->X_B->table;
 					    }
 					    gpower=(int)(tchan->Xpower+1e-6);
+					    if (tchan->instant & INSTANTX)
+							    gpower=-gpower;
 					}
 					break;
 				    case 1:   /* Y gate */
@@ -1141,6 +1037,8 @@ int h_hh_chip_init(hsolve)
 					    table=tchan->Y_B->table;
 					}
 					gpower=(int)(tchan->Ypower+1e-6);
+					if (tchan->instant & INSTANTY)
+							    gpower=-gpower;
 					break;
 				   case 2:   /* Z gate */
 					if (!tchan->Z_alloced) continue;
@@ -1152,6 +1050,8 @@ int h_hh_chip_init(hsolve)
 					    table=tchan->Z_B->table;
 					}
 					gpower=(int)(tchan->Zpower+1e-6);
+					if (tchan->instant & INSTANTZ)
+							    gpower=-gpower;
 					break;
 				}
 				switch (gindex) {
@@ -1183,7 +1083,7 @@ int h_hh_chip_init(hsolve)
 					break;
 				    }
 				}
-				if (k==nvtables) printf("Error in %s: Bug #5 in h_hh_chip_init %d.\n",hsolve->name,table);
+				if (k==nvtables) printf("Error in %s: Bug #5 in h_hh_chip_init %d.\n",hsolve->name, (ADDR) table);
 				ops[nop+1]=gpower;
 				nop+=2;
 				if (ct<TABCURR_T) nchip++;	/* X, Y or Z value */
@@ -1206,17 +1106,18 @@ int h_hh_chip_init(hsolve)
 			    ops[nop+1]=cp;	/* stablist index */
 			    ops[nop+2]=-1;	/* initialize as no-event */
 			    ops[nop+3]=j;	/* pointer to child */
-			    h_check_msgs(hsolve,child,SYN2_OP,i,nop+1,nchip);
+			    h_check_msgs(hsolve,child,SYN2_OP,ct,i,nop+1,nchip);
 			    nop+=4;	/* SYN2/3_OP,stablist_index,counter,child_ind */
 			    nchip+=2;       /* X and Y */
 			    if (childtypes[j+1]==MGBLOCK_T) {
-				h_check_msgs(hsolve,children[j+1],MGBLOCK_OP,i,nop,nchip);
+				h_check_msgs(hsolve,children[j+1],MGBLOCK_OP,MGBLOCK_T,i,nop,nchip);
 				childops[j+1]=nop;
 				childchips[j+1]=nchip;
 				ops[nop]=MGBLOCK_OP;
 				nop++;
 				nchip+=3;   /* KMg_A 1/KMg_B and CMg */
 				j++;
+				cl=childlink[j]; /* for flux from Mg_block */
 			    }
 			}
 			/* take care of flux storage */
@@ -1247,7 +1148,7 @@ int h_hh_chip_init(hsolve)
 			break;
 		
 		    case NERNST_T:
-			h_check_msgs(hsolve,child,NERNST_OP,i,nop,nchip);
+			h_check_msgs(hsolve,child,NERNST_OP,ct,i,nop,nchip);
 			ops[nop]=NERNST_OP;
 			nop++;
 			nchip+=2;	/* constant and gmax */
@@ -1255,7 +1156,7 @@ int h_hh_chip_init(hsolve)
 			break;
 
 		    case GHK_T:
-			h_check_msgs(hsolve,child,GHK_OP,i,nop,nchip);
+			h_check_msgs(hsolve,child,GHK_OP,ct,i,nop,nchip);
 			ops[nop]=GHK_OP;
 			nop++;
 			nchip+=3;	/* zF/RT, -ZF and gmax */
@@ -1269,6 +1170,16 @@ int h_hh_chip_init(hsolve)
 	    nchip+=2;	/* tybc and diagterm fields for COMPT_OP or LCOMPT_OP */
 	}	/* i loop */
 	ops[nop]=LCOMPT_OP;
+
+	/*- free compartment flags */
+
+	if (hsolve->piCompFlags)
+	{
+	    free(hsolve->piCompFlags);
+
+	    hsolve->piCompFlags = NULL;
+	}
+
 	if (nop>=hsolve->nops) {
 		printf("Error in %s: Bug #1 in h_hh_chip_init %d.\n",hsolve->name,nop);
 	}
@@ -1285,6 +1196,7 @@ int duplicate_tables(hsolve)
 {
 	int i,j,k,l,n,empty2Dtabcurr,oldl;
 	int tabsize;
+	int *tabinteg;
 	double	x,dx;
 	double  dt,tby2,Gk;
 	double	*table,**table2,*htable,*gtable,**atable2,**btable2,*xvals;
@@ -1296,12 +1208,8 @@ int duplicate_tables(hsolve)
 	tby2 = dt/2.0;
 	hsolve->tables=(double **)calloc(nvtables,sizeof(double *));
 	hsolve->tabcols=(int *)calloc(nvtables,sizeof(int));
+	hsolve->tabinteg=(int *)calloc(nvtables,sizeof(int));
 	if (hsolve->ntab) {
-	    for(i=0;i<nvtables;i++) {
-		hsolve->tables[i]=tablist[i];
-		hsolve->tabcols[i]=tabcols[i];
-	    }
-
 	    hsolve->ncols=ncols;
 	    if (hsolve->readflag>=HSETUP_T) {	/* copy and store the data */
 		tabsize=hsolve->xdivs+1;
@@ -1327,7 +1235,10 @@ int duplicate_tables(hsolve)
 		for(i=0; i<nvtables; i++) {	
 		    if (tabcols[i]==1) {	/* 1D */
 			tabindex[i]=l;
-			if (tabintegrate[i]) {
+			hsolve->tables[l]=tablist[i];
+			hsolve->tabcols[l]=tabcols[i];
+			hsolve->tabinteg[l]=tabintegrate[i];
+			if (tabintegrate[i]>0) {
 			    /* B table */
 			    table=tablist[i];
 			    for(j=0,k=l;j<tabsize;j++,k+=ncols) {
@@ -1341,20 +1252,27 @@ int duplicate_tables(hsolve)
 				htable[k]=dt*table[j];
 			    }
 			    l++;
-			} else {	/* TABCURR_T */
-			    /* G_tab */
+			} else {	/* TABCURR_T or instant gate */
+			    /* G_tab or B table: copy unaltered */
 			    gtable=tablist[i];
 			    for(j=0,k=l;j<tabsize;j++,k+=ncols) {
 				htable[k]=gtable[j];
 			    }
 			    l++;
-			    /* I_tab: convert to an E_tab containing Ek values*/
 			    i++;	/* this table also */
 			    table=tablist[i];
-			    for(j=0,k=l;j<tabsize;j++,k+=ncols) {
-				Gk=gtable[j];
-				/* Ek=(Ik + Gk*Vm)/Gk */
-				htable[k]=(table[j] + Gk*xvals[j])/Gk;
+			    if (tabintegrate[i]==-1) { /* instant gate */
+				/* A table: copy unaltered */
+				for(j=0,k=l;j<tabsize;j++,k+=ncols) {
+				    htable[k]=table[j];
+				}
+			    } else {	/* TABCURR_T */
+				/* I_tab: convert to an E_tab containing Ek */
+				for(j=0,k=l;j<tabsize;j++,k+=ncols) {
+				    Gk=gtable[j];
+				    /* Ek=(Ik + Gk*Vm)/Gk */
+				    htable[k]=(table[j] + Gk*xvals[j])/Gk;
+				}
 			    }
 			    l++;
 			}
@@ -1363,10 +1281,13 @@ int duplicate_tables(hsolve)
 		/* then all 2D tables */
 		for(i=0; i<nvtables; i++) {	
 		    if (tabcols[i]>1) {		/* 2D */
+			hsolve->tables[l]=tablist[i];
+			hsolve->tabcols[l]=tabcols[i];
+			hsolve->tabinteg[l]=tabintegrate[i];
 			tabindex[i]=l;
 			btable2=(double **)tablist[i];
 			atable2=(double **)tablist[i+1];
-			if (tabintegrate[i]) {
+			if (tabintegrate[i]>0) {
 			    for (n=0; n<tabcols[i]; n++) {
 				/* B table */
 				for(j=0,k=l;j<tabsize;j++,k+=ncols) {
@@ -1380,14 +1301,19 @@ int duplicate_tables(hsolve)
 				l++;
 			    }
 			    i++;	/* we did 2 tables */
-			} else {	/* TABCURR_T */
+			} else {	/* TABCURR_T or instant gate */
 			    for (n=0; n<tabcols[i]; n++) {
 				for(j=0,k=l;j<tabsize;j++,k+=ncols) {
-				    /* G_tab */
+				    /* G_tab or B table: copy unaltered */
 				    Gk=htable[k]=btable2[j][n];
-				    /* I_tab: convert to E_tab with Ek values */
-				    /* Ek=(Ik + Gk*Vm)/Gk */
-				    htable[k+1]=(atable2[j][n] +Gk*xvals[j])/Gk;
+				    if (tabintegrate[i]==-1) { /* instant gate*/
+					/* A table: copy unaltered */
+					htable[k]=atable2[j][n];
+				    } else {	/* TABCURR_T */
+					/* I_tab: convert to E_tab with Eks */
+					/* Ek=(Ik + Gk*Vm)/Gk */
+					htable[k+1]=(atable2[j][n] +Gk*xvals[j])/Gk;
+				    }
 				}
 				l+=2;
 			    }
@@ -1412,36 +1338,37 @@ do_chip_hreset(hsolve,actionstr)
 	char    *actionstr;
 {
 	int 	ncompts=hsolve->ncompts;
-	Element	**children,**compts;
-	int 	i,j,k,n;
-	short   *childtypes;
-	int	*elmnum,*childstart,*childlink,*childmsg;
+	Element **compts=hsolve->compts;
+	int     *elmnum=hsolve->elmnum;
+	Element **children=hsolve->children;
+	short   *childtypes=hsolve->childtypes;
+	int     *childlink=hsolve->childlink;
+	int     *childmsg=hsolve->childmsg;
+	int     *childstart=hsolve->childstart;
+	int	*tabinteg=hsolve->tabinteg;
+	int	*tabcols=hsolve->tabcols;
+	double	*tablist=hsolve->tablist;
+	int	ncols=hsolve->ncols;
+	int	tabsize=hsolve->xdivs+1;
+	int 	i,j,k,l,n;
 	int	ct,cl,cm;
 	double  dt,ratio;
 	Action  *action;
 
 	if ((hsolve->readflag==HSETUP_T)&&(IsSilent()<0))
 		printf("transferring element field values into solve arrays\n");
-	compts=hsolve->compts;
-	elmnum=hsolve->elmnum;
-	children=hsolve->children;
-	childtypes=hsolve->childtypes;
-	childlink=hsolve->childlink;
-	childmsg=hsolve->childmsg;
-	childstart=hsolve->childstart;
 
 	action = GetAction(actionstr);
 	dt=Clockrate(hsolve);
 	if (dt != hsolve->dt) {
 	    /* user changed the clocks after SETUP call */
 	    ratio=dt/hsolve->dt;
-	    for(i=0;i<hsolve->ntab;i++) {
-		for(j=0,k=i;j<hsolve->xdivs+1;j++,k+=hsolve->ntab) {
-		    hsolve->tablist[k]=1.0+((hsolve->tablist[k]-1.0)*ratio);
-		}
-		i++;
-		for(j=0,k=i;j<hsolve->xdivs+1;j++,k+=hsolve->ntab) {
-		    hsolve->tablist[k]=hsolve->tablist[k]*ratio;
+	    for(i=0;i<hsolve->ntab;i=i+2) {
+		if (tabinteg[i]>0) {
+		    for (j=0,k=i;j<tabsize;j++,k+=ncols) {
+			tablist[k]=1.0+((tablist[k]-1.0)*ratio);
+			tablist[k+1]=tablist[k+1]*ratio;
+		    }
 		}
 	    }
 	    hsolve->dt = dt;
@@ -1492,13 +1419,13 @@ do_chip_hreset(hsolve,actionstr)
 			break;
 
 		    case NERNST_T:
-			chip_put_nernst(hsolve,j,cl);
+			chip_put_nernst(hsolve,j,cm);
 			/* this overwrites conc arrays, but if necessary, conc 
 			**  will be reset next */
 			break;
 
 		    case GHK_T:
-			chip_put_ghk(hsolve,j,cl);
+			chip_put_ghk(hsolve,j,cm);
 			/* this overwrites conc arrays, but if necessary, conc 
 			**  will be reset next */
 			break;
@@ -1508,11 +1435,16 @@ do_chip_hreset(hsolve,actionstr)
 			chip_put_Ca_concen(hsolve,i,j);
 			break;
 
+		    case POOL_T:
+			chip_put_pool(hsolve,j,cm);
+			chip_put_flux(hsolve,i,j);
+			break;
+
 		    case DIFSHELL_T:
 		    case DIFSLAB_T:
 		    case DIFUSER_T:
 			chip_put_shell(hsolve,j,cm);
-			chip_put_difshell(hsolve,i,j);
+			chip_put_flux(hsolve,i,j);
 			break;
 
 		    case FIXBUFF_T:
@@ -1534,7 +1466,15 @@ do_chip_hreset(hsolve,actionstr)
 
 		    case MMPUMP_T:
 		    case ELMMPUMP_T:
-			chip_put_mmpump(hsolve,j,cm);
+			chip_put_mmpump(hsolve,j,cl);
+			break;
+
+		    case HILLPUMP_T:
+			chip_put_hillpump(hsolve,j,cm);
+			break;
+
+		    case POOLBUFF_T:
+			chip_put_poolbuffer(hsolve,j,cm);
 			break;
 
 		    default:
@@ -1591,23 +1531,28 @@ do_chip_hsave(hsolve)
 			break;
 
 		    case NERNST_T:
-			chip_get_nernst(hsolve,j,cl);
+			chip_get_nernst(hsolve,j,cm);
 			break;
 		    
 		    case GHK_T:
-			chip_get_ghk(hsolve,j,cl);
+			chip_get_ghk(hsolve,j,cm);
 			break;
 
 		    case CACONCEN_T:
 			chip_get_concen(hsolve,j,cm);
 			break;
 
+		    case POOL_T:
+			chip_put_pool(hsolve,j,cm);
+			break;
+
 		    case DIFSHELL_T:
 		    case DIFSLAB_T:
 		    case DIFUSER_T:
 			chip_get_shell(hsolve,j,cm);
 			break;
 
+		    case POOLBUFF_T:
 		    case FIXBUFF_T:
 		    case BUFSHELL_T:
 		    case BUFSLAB_T:
@@ -1630,7 +1575,7 @@ do_chip_hsave(hsolve)
 			break;
 
 		    default:
-			    break;
+			break;
 		}
 	    }	/* j loop */
 	}	/* i loop */
