@@ -1,6 +1,9 @@
 /*
-** $Id: newconn_ext.h,v 1.3 1996/07/22 23:52:54 dhb Exp $
+** $Id: newconn_ext.h,v 1.4 2001/04/25 17:16:59 mhucka Exp $
 ** $Log: newconn_ext.h,v $
+** Revision 1.4  2001/04/25 17:16:59  mhucka
+** Misc. small changes to improve portability and address compiler warnings.
+**
 ** Revision 1.3  1996/07/22 23:52:54  dhb
 ** Changes from Mike Vanier:
 **   Generic handling of synchan types
@@ -17,6 +20,7 @@
 #include "newconn_defs.h"
 #include "newconn_struct.h"
 #include "synaptic_event.h"
+#include "shell_func_ext.h"
 #include "seg_struct.h"  /* for compartment definition */
 
 extern float    rangauss();
