@@ -1,7 +1,10 @@
 /*
-** $Id: olf_ext.h,v 1.2 1997/05/29 08:30:59 dhb Exp $
+** $Id: olf_ext.h,v 1.3 2001/04/25 17:16:59 mhucka Exp $
 **
 ** $Log: olf_ext.h,v $
+** Revision 1.3  2001/04/25 17:16:59  mhucka
+** Misc. small changes to improve portability and address compiler warnings.
+**
 ** Revision 1.2  1997/05/29 08:30:59  dhb
 ** Update from Antwerp GENESIS version 21e
 **
@@ -14,6 +17,7 @@
 */
 
 #include "sim_ext.h"
+#include "shell_func_ext.h"
 #include "olf_struct.h"
 
 extern double TabInterp();
