@@ -1,7 +1,10 @@
-static char rcsid[] = "$Id: out_script.c,v 1.1 1992/12/11 19:03:27 dhb Exp $";
+static char rcsid[] = "$Id: out_script.c,v 1.2 2000/09/11 15:50:55 mhucka Exp $";
 
 /*
 ** $Log: out_script.c,v $
+** Revision 1.2  2000/09/11 15:50:55  mhucka
+** Several misc. fixes from Hugo Cornelis.
+**
 ** Revision 1.1  1992/12/11 19:03:27  dhb
 ** Initial revision
 **
@@ -13,6 +16,8 @@ ScriptOutput(script,action)
 struct script_output_type *script;
 Action		*action;
 {
+    int iResult = 1;
+
     SELECT_ACTION(action){
     case PROCESS:
 	/*
@@ -23,12 +28,35 @@ Action		*action;
 	}
 	break;
     case SET:
-	if(strcmp(action->argv[0],"command") == 0){
+	if(strcmp(action->argv[0],"command") == 0)
+	{
+	    /*- free old command */
+
+	    if (script->command)
+		free(script->command);
+
 	    script->command = CopyString(action->argv[1]);
-	} else {
-	    printf("field '%s' can not be set by the user\n",action->argv[0]);
-	    return;
+
+	    /*
+	    ** remember : field set by object
+	    */
+
+	    iResult = 1;
 	}
+	else
+	{
+	    /*
+	    ** remember : field should be set by kernel
+	    */
+
+	    iResult = 0;
+
+	    break;
+	}
+/* 	} else { */
+/* 	    printf("field '%s' can not be set by the user\n",action->argv[0]); */
+/* 	    return(iResult); */
+/* 	} */
     case RESET:
     case RECALC:
 	if(script->command){
@@ -41,5 +69,7 @@ Action		*action;
 	Actionname(action));
 	break;
     }
+
+    return(iResult);
 }
 
