@@ -1,7 +1,11 @@
-static char rcsid[] = "$Id: shell_debug.c,v 1.3 1997/07/18 02:55:46 dhb Exp $";
+static char rcsid[] = "$Id: shell_debug.c,v 1.4 2000/04/24 07:54:36 mhucka Exp $";
 
 /*
 ** $Log: shell_debug.c,v $
+** Revision 1.4  2000/04/24 07:54:36  mhucka
+** Moved the definition of variable "debug" out of shell.h and into this
+** file.  .h files should not define variables.
+**
 ** Revision 1.3  1997/07/18 02:55:46  dhb
 ** Fix to getopt problem.  getopt(), optopt and optind are now
 ** G_getopt(), G_optopt and G_optind.
@@ -16,6 +20,9 @@ static char rcsid[] = "$Id: shell_debug.c,v 1.3 1997/07/18 02:55:46 dhb Exp $";
 
 #include "shell_ext.h"
 
+int debug = 0;
+
+
 int do_debug(argc,argv)
 int	argc;
 char	**argv;
