@@ -1,7 +1,10 @@
-static char rcsid[] = "$Id: sim_buffer.c,v 1.2 1993/07/21 21:31:57 dhb Exp $";
+static char rcsid[] = "$Id: sim_buffer.c,v 1.3 2001/04/25 17:17:01 mhucka Exp $";
 
 /*
 ** $Log: sim_buffer.c,v $
+** Revision 1.3  2001/04/25 17:17:01  mhucka
+** Misc. small changes to improve portability and address compiler warnings.
+**
 ** Revision 1.2  1993/07/21 21:31:57  dhb
 ** fixed rcsid variable type
 **
@@ -10,6 +13,8 @@ static char rcsid[] = "$Id: sim_buffer.c,v 1.2 1993/07/21 21:31:57 dhb Exp $";
  *
 */
 
+#include <stdio.h>
+#include "shell_func_ext.h"
 #include "sim_ext.h"
 
 DeleteBuffer(buffer)
