@@ -1,13 +1,24 @@
-/* $Id: sim_dump.c,v 1.7 1997/07/24 00:24:18 dhb Exp $ */
-/* $Log: sim_dump.c,v $
- * Revision 1.7  1997/07/24 00:24:18  dhb
- * Fixed from PSC:
- *   Missing arg to DumpObj() added.
- *   Move decl for do_add_msg() to global scope
- *
- * Revision 1.6  1997/07/18 02:58:25  dhb
- * Fix for getopt problem; getopt(), optopt and optind are now
- * G_getopt(), G_optopt and G_optind.
+/* $Id: sim_dump.c,v 1.14 2001/06/29 21:27:37 mhucka Exp $ */
+/*
+** $Log: sim_dump.c,v $
+** Revision 1.14  2001/06/29 21:27:37  mhucka
+** Fixes from Hugo C.
+**
+** Revision 1.13  2001/04/25 17:17:01  mhucka
+** Misc. small changes to improve portability and address compiler warnings.
+**
+** Revision 1.11  2000/06/12 04:52:17  mhucka
+** 1) Added a missing typecast in DumpMessages().
+** 2) Added NOTREACHED comments where appropriate.
+**
+** Revision 1.10  1998/08/28 17:13:25  dhb
+** Added options processing code for do_substitute_info().
+**
+** Revision 1.9  1998/07/21 18:58:50  dhb
+** Fixed ANSI C isms.
+**
+ * Revision 1.8  1998/07/15 06:52:26  dhb
+ * Upi update
  *
  * Revision 1.5  1995/09/26 22:00:44  dhb
  * Changes from Upi Bhalla:
@@ -80,6 +91,8 @@
  * Initial revision
  * */
 
+#include <stdio.h>
+#include "shell_func_ext.h"
 #include "sim_ext.h"
 #include "hash.h"
 
@@ -132,6 +145,28 @@ struct dumplist_type {
 	GenesisObject	*object;
 };
 
+struct obj_substitute_type {
+	char *orig;
+	char *new;
+};
+
+struct msg_substitute_type {
+	char *destobj;
+	char *orig_msgtype;
+	int argc;
+	char *argv[10];
+};
+
+#define N_SUBSTITUTE 100
+
+static int n_obj_substitute = 0;
+static int n_msg_substitute = 0;
+static struct obj_substitute_type obj_substitute[N_SUBSTITUTE];
+static struct msg_substitute_type msg_substitute[N_SUBSTITUTE];
+static int msg_substitution = 0;
+static int targc = 0;
+static char *targv[10];
+
 #define MSGDUPELISTSIZE 100
 
 static struct dumplist_type *DumpList = NULL;
@@ -159,11 +194,17 @@ static int DumpMsgIn();
 static char *FindFieldName();
 static int DumpMsgFields();
 static void DumpElements();
-static int dump_hash_function();
+static ADDR dump_hash_function();
 static ENTRY *dump_hash_find();
 static ENTRY *dump_hash_enter();
-int do_add_msg();
 int do_dump_add_msg(); /* Forward declaration */
+#ifdef __STDC__
+char* do_dump_call(int argc, char** argv); /* Forward declaration */
+char* do_call_element(int argc, char** argv); /* Original declaration*/
+#else
+char* do_dump_call(); /* Forward declaration */
+char* do_call_element(); /* Original declaration*/
+#endif
 
 do_simobjdump(argc,argv)
 	int 	argc;
@@ -305,7 +346,7 @@ int AddObjToDumpList(objname,fields,nfields,use_coords,do_dump_action)
 */
 	nfields = k;
 
-	if (dumpitem = FindOnDumpList(object)) {
+	if ((dumpitem = FindOnDumpList(object))) {
 		/* Get rid of last entry */
 		if (dumpitem->nfields > 0) {
 			int j;
@@ -375,6 +416,7 @@ static int FilterStr(str,retstr,do_quote)
 			case '\0':
 				newstr[j] = curr;
 				return(j);
+				/* NOTREACHED */
 			break;
 			default:
 				newstr[j] = curr;
@@ -467,7 +509,7 @@ int DumpObj(fp,objname,fields,nfields,use_coords)
 	fprintf(fp,"%s\n",dumpline);
 }
 
-do_initdump(argc,argv)
+void do_initdump(argc,argv)
 	int 	argc;
 	char	**argv;
 {
@@ -519,6 +561,8 @@ do_initdump(argc,argv)
 		MsgDupElist->nelements = 0;
 		AddFunc("addmsg", do_dump_add_msg, "void");
 	}
+	/* Replace the normal 'call' command with do_dump_call */
+	AddFunc("call", do_dump_call, "char*");
 }
 
 do_enddump(argc,argv)
@@ -546,9 +590,116 @@ do_enddump(argc,argv)
 		MsgDupElist->nelements = 0;
 		AddFunc("addmsg", do_add_msg, "void");
 	}
+	AddFunc("call", do_call_element, "char*");
 	AllowMsgDup = 0;
 }
 
+/* Adds objects to obj_substitute list */
+#ifdef __STDC__
+do_obj_substitute(int argc, char** argv) {
+#else
+do_obj_substitute(argc, argv)
+int argc;
+char** argv;
+{
+#endif
+	int i;
+
+	initopt(argc,argv, "orig_obj_name new_obj_name");
+	if (G_getopt(argc,argv) != 0) {
+		printoptusage(argc,argv);
+		return;
+	}
+	for (i = 0; i < n_obj_substitute; i++) {
+		if (strcmp(optargv[1], obj_substitute[i].orig) == 0) {
+			Error();
+			printf("Duplicating object substitution for %s, %s\n",
+				optargv[1], optargv[2]);
+			return;
+		}
+	}
+	if (n_obj_substitute >= N_SUBSTITUTE) {
+		Error();
+		printf("Exceeded max number of object substitutes %d\n",
+			N_SUBSTITUTE);
+	}
+	obj_substitute[n_obj_substitute].orig = CopyString(optargv[1]);
+	obj_substitute[n_obj_substitute].new = CopyString(optargv[2]);
+	n_obj_substitute++;
+}
+
+/* Add msgs to msg_substitute list. Note that if we
+** are also doing obj substitution, we must ensure that we use
+** the _substituted_ name of the dest obj is used.
+*/
+#ifdef __STDC__
+do_msg_substitute(int argc, char** argv) {
+#else
+do_msg_substitute(argc, argv)
+int argc;
+char** argv;
+{
+#endif
+	int i;
+
+	initopt(argc,argv, "destobj orig_msg_type new_msg_type field1 ...");
+	if (G_getopt(argc,argv) != 0) {
+		printoptusage(argc,argv);
+		return;
+	}
+	for (i = 0; i < n_msg_substitute; i++) {
+		if (strcmp(optargv[1], msg_substitute[i].destobj) == 0 &&
+			strcmp(optargv[2], msg_substitute[i].orig_msgtype) == 0) {
+			Error();
+			printf("Duplicating msg substitution for %s, %s\n",
+				optargv[1], optargv[2]);
+			return;
+		}
+	}
+	if (n_msg_substitute >= N_SUBSTITUTE) {
+		Error();
+		printf("Exceeded max number of msg substitutes %d\n",
+			N_SUBSTITUTE);
+	}
+	msg_substitute[n_msg_substitute].destobj = CopyString(optargv[1]);
+	msg_substitute[n_msg_substitute].orig_msgtype =
+		CopyString(optargv[2]);
+	for (i = 0; i < optargc - 3; i++) {
+		msg_substitute[n_msg_substitute].argv[i] =
+			CopyString(optargv[i + 3]);
+	}
+	msg_substitute[n_msg_substitute].argc = optargc - 3;
+	n_msg_substitute++;
+}
+
+/* Just an info function on substitutions */
+do_substitute_info(argc,argv) {
+	int i,j;
+
+	initopt(argc, argv, "");
+	if (G_getopt(argc, argv) != 0) {
+		printoptusage(argc, argv);
+		return;
+	}
+
+	printf("Number of object substitutions: %d\n", n_obj_substitute);
+	for(i = 0; i < n_obj_substitute; i++) {
+		printf("%d	orig=%s		new=%s\n",i,
+			obj_substitute[i].orig, obj_substitute[i].new);
+	}
+
+	printf("\nNumber of msg substitutions: %d\n", n_msg_substitute);
+	printf("No.	Obj type	Orig msgtype	New msg type	fields\n");
+	for(i = 0; i < n_msg_substitute; i++) {
+		printf("%d	%s		%s		%s	",i,
+			msg_substitute[i].destobj, msg_substitute[i].orig_msgtype,
+			msg_substitute[i].argv[0]);
+		for (j = 1; j < msg_substitute[i].argc; j++) 
+			printf("	%s",msg_substitute[i].argv[j]);
+		printf("\n");
+	}
+}
+
 /* Finds the dump item corresponding to a given object */
 static struct dumplist_type *FindOnDumpList(obj)
 	GenesisObject *obj;
@@ -590,7 +741,7 @@ do_simdump(argc,argv)
 	while ((status = G_getopt(argc,argv)) == 1) {
 		if (strcmp(G_optopt,"-tree") == 0) {
 			Element	*tree;
-			if (tree = GetElement(optargv[1])) {
+			if ((tree = GetElement(optargv[1]))) {
 			} else {
 				Error();
 				printf("Cannot find tree %s\n",optargv[1]);
@@ -745,8 +896,7 @@ WriteDumpList(fp)
 	for(i = 0 ; i < DumpListSize; i++)
 		DumpObj(fp,DumpList[i].object->name,
 			DumpList[i].fields,
-			DumpList[i].nfields,
-			0);
+			DumpList[i].nfields, FALSE);
 }
 
 static void DumpElements(fp,elist)
@@ -812,13 +962,11 @@ static void DumpElements(fp,elist)
 
 /* This is a duplication of a function in xo. I think we should
 ** add this more sensible set of hash utilities to the sys library */
-static int dump_hash_function(key,table)
+static ADDR dump_hash_function(key,table)
 	char	*key;
 	HASH	*table;
 {
-	int val = (int)key;
-
-	return(((val & 0xff) + (val >> 8)) % table->size);
+	return((((ADDR)key & 0xff) + ((ADDR)key >> 8)) % (ADDR)table->size);
 }
 
 static ENTRY *dump_hash_find(key,table)
@@ -875,7 +1023,7 @@ char *key;
     /*
     ** leave the last entry empty to identify the end of the table
     */
-    bcopy(item,hash_ptr,sizeof(ENTRY));
+    BCOPY(item,hash_ptr,sizeof(ENTRY));
     return(hash_ptr);
 }
 
@@ -921,7 +1069,7 @@ static int DumpMessages(fp,elist,do_all)
 			msg = elm->msgin;
 			for(count = 0 ; count < elm->nmsgin; count++) {
 				/* check if the msg src is on the hash table */
-				if (tempentry = dump_hash_find(msg->src,hash_table)) {
+				if ((tempentry = dump_hash_find((char *) msg->src,hash_table))) {
 					DumpMsgIn(fp,msg,elm);
 				}
 				msg = MSGINNEXT(msg);
@@ -980,14 +1128,14 @@ static int DumpMsgFields(dumpline, msg)
 	slot = (Slot *)(msg + 1);
 	for(i= 0; i < (int)msg->nslots; i++) {
 		/* check if it is a string msg slot */
-		if (slotfield = GetStringMsgData(msg,i)) {
+		if ((slotfield = GetStringMsgData(msg,i))) {
 			strcat(dumpline,"*");
 			strcat(dumpline,slotfield);
 			strcat(dumpline," ");
 		} else { /* no, we need to look for a field */
 			char *fname;
 			fieldoffset = slot[i].data - (char *)elm;
-			if (fname = FindFieldName(elm->object,fieldoffset)) {
+			if ((fname = FindFieldName(elm->object,fieldoffset))) {
 				strcat(dumpline,fname);
 				strcat(dumpline," ");
 			}
@@ -1001,17 +1149,18 @@ static char *FindFieldName(object,offset)
 {
 	char	*str;
 	char	*tmp;
-	char	fieldlist[1000];
+	char	*fieldlist;
 	static char	fieldname[50];
 	char	fullname[100];
-	extern char	*FieldHashFind();
+	extern char	*FieldHashFindAndCopy();
 	Info	info;
 
 	/* in str the fieldnames are separated by '\n'
 	** so we make a copy of str and munge through it
 	** to get the individual names */
-	if (str = FieldHashFind(object->type)) {
-		strcpy(fieldlist,str);
+	if ((str = FieldHashFindAndCopy(object->type))) {
+		/* strcpy(fieldlist,str); */
+		fieldlist = str;
 		for(str = fieldlist; *str != '\0'; ) {
 			tmp = str;
 			while (*tmp != '\n') tmp++;
@@ -1020,11 +1169,14 @@ static char *FindFieldName(object,offset)
 			if (GetInfo(fullname,&info)) {
 				if (info.offset == offset) {
 					strcpy(fieldname,str);
+					free(str);
 					return(fieldname);
 				}
 			}
 			str = tmp+1;
 		}
+
+		free(str);
 	} else {
 	}
 	return("x"); /* safe fallback field name */
@@ -1049,6 +1201,28 @@ static char *ParseString(str)
 	return newstr;
 }
 
+/* Here we scan the list of objects to be substituted to see if the
+** requested object needs to be changed */
+#ifdef __STDC__
+int ObjectSubstitute(char* objname, char** retname) {
+#else
+int ObjectSubstitute(objname, retname)
+char* objname;
+char** retname;
+{
+#endif
+	int i;
+
+	for (i = 0; i < n_obj_substitute; i++) {
+		if (strcmp(objname, obj_substitute[i].orig) == 0) {
+			*retname = obj_substitute[i].new;
+			return 1;
+		}
+	}
+	*retname = objname;
+	return 0;
+}
+
 do_simundump(argc,argv)
 	int 	argc;
 	char	**argv;
@@ -1058,13 +1232,14 @@ do_simundump(argc,argv)
 	char	*targv[3];
 	int		status;
 	struct dumplist_type *dumpitem;
+	int orig_optargc;
+	int subst_flag = 0;
 
 	initopt(argc,argv, "object element ... -tree tree x y z");
 	while ((status = G_getopt(argc,argv)) == 1) {
 		if (strcmp(G_optopt,"-tree") == 0) {
 			Element	*tree;
-			if (tree = GetElement(optargv[1])) {
-			} else {
+			if (!(tree = GetElement(optargv[1]))) {
 				Error();
 				printf("Cannot find tree %s\n",optargv[1]);
 				return;
@@ -1077,9 +1252,10 @@ do_simundump(argc,argv)
 		printoptusage(argc,argv);
 		return;
 	}
+	orig_optargc = optargc;
 
 	/* check if the element has already been created. */
-	if (elm = GetElement(optargv[2])) {
+	if ((elm = GetElement(optargv[2]))) {
 		if (strcmp(elm->object->name,optargv[1]) != 0) {
 			Error();
 			printf("%s: element %s exists but is not of type %s\n",
@@ -1106,9 +1282,12 @@ do_simundump(argc,argv)
 	/* create the element. It is simplest to use the usual 'create'
 	** command here */
 		targv[0] = "simundump";
-		targv[1] = optargv[1];
+	/* Here we check if object substitution is enabled */
+		subst_flag = ObjectSubstitute(optargv[1], &targv[1]);
 		targv[2] = optargv[2];
-		do_create(3,argv);
+	/* Nasty surprise here: if the arguments are changed, then
+	** subsequent values of optargv and optargc are also changed. Ugh */
+		do_create(3,targv);
 		elm = RecentElement();
 		if (!elm) return;
 	}
@@ -1124,28 +1303,41 @@ do_simundump(argc,argv)
 		return;
 	}
 
-	if (DumpVersion < 3)
-		i = optargc - 3;
-	else
-		i = optargc - 4;
+	/* Here we eliminate all fields, except the flags, if substitution
+	** is enabled */
+	if (subst_flag) {
+		if (DumpVersion < 3)
+			orig_optargc = 3;
+		else
+			orig_optargc = 4;
+	}
 
-	if (dumpitem->nfields != i) {
+	if (DumpVersion < 3)
+		i = orig_optargc - 3;
+	else
+		i = orig_optargc - 4;
+
+	/* We bypass this check if object substitution has occurred,
+	** because the fields are unknown */
+	if (dumpitem->nfields != i && !subst_flag) {
 		Error();
 		printf("dump fields for object '%s' do not match data list\n",
 			elm->object->name);
 		return;
 	}
+	/* We need to use argv rather than optargv, because 
+	** optargv changes sometimes in the do_create function */
 	if (DumpVersion < 3) {
-		for (i = 3; i < optargc; i++) {
-			SetElement(elm,dumpitem->fields[i - 3],optargv[i]);
+		for (i = 3; i < orig_optargc; i++) {
+			SetElement(elm,dumpitem->fields[i - 3],argv[i]);
 			/* The SetElement issues its own warnings */
 		}
 	} else {
 		/* Load in the flags of the element */
-		elm->flags = atoi(optargv[3]);
-		for (i = 4; i < optargc; i++) {
+		elm->flags = atoi(argv[3]);
+		for (i = 4; i < orig_optargc; i++) {
 			SetElement(elm,dumpitem->fields[i - 4],
-				ParseString(optargv[i]));
+				ParseString(argv[i]));
 			/* The SetElement issues its own warnings */
 		}
 	}
@@ -1296,6 +1488,66 @@ Element     *element;
     return(1);
 }
 
+/* This function checks if the specified msg is to be substituted,
+** and if so it sets up the alternative arguments for the msg.
+** Note that at this point the dest obj may have been converted.
+** We have options:
+**	- just check for the converted object, and assume that the
+**		converted object will always be of a new class.
+**	- convert back to the original object for checks. Again, we
+**		must assume that the converted object will be of a new clas.
+*/
+#ifdef __STDC__
+MessageSubstitute(Element *dest, char* msgtype, int argc, char** argv) {
+#else
+MessageSubstitute(dest, msgtype, argc, argv)
+Element *dest;
+char* msgtype;
+int argc;
+char** argv;
+{
+#endif
+	char *destobj = dest->object->name;
+	int i, j;
+	
+	for (i = 0; i < n_msg_substitute; i++) {
+		if (strcmp(msgtype, msg_substitute[i].orig_msgtype) == 0) {
+			if (strcmp(destobj, msg_substitute[i].destobj) == 0) {
+				msg_substitution = 1;
+				targc = msg_substitute[i].argc + 3;
+				targv[0] = argv[0];
+				targv[1] = argv[1];
+				targv[2] = argv[2];
+				for (j = 3; j < targc; j++) {
+					if (strcmp(msg_substitute[i].argv[j-3],".") == 0)
+						targv[j] = argv[j];
+					else
+						targv[j] = msg_substitute[i].argv[j-3];
+				}
+				return;
+			}
+		}
+	}
+	msg_substitution = 0;
+}
+
+
+/* This simple function switches between arguments if msg_substitution
+** is on */
+#ifdef __STDC__
+int add_msg_wrapper(int argc, char **argv) {
+#else
+int add_msg_wrapper(argc, argv)
+int argc;
+char **argv;
+{
+#endif
+	if (!msg_substitution)
+		return do_add_msg(argc,argv);
+	
+	return do_add_msg(targc,targv);
+}
+
 /* This is basically a shell for the normal addmsg command.
 ** It is designed to prevent duplication of existing msgs when
 ** a dumpfile is loaded on top of an existing model.
@@ -1330,10 +1582,13 @@ int do_dump_add_msg(argc,argv)
 	if (!dest) /* Cant do anything anyway ! */
 		return 0;
 
+	/* Here we change things around if MessageSubstitution is active */
+	MessageSubstitute(dest,optargv[3],argc,argv);
+
 	/* If there are no elements on the MsgDupElist, just call
 	** the usual do_add_msg function */
 	if (MsgDupElist->nelements == 0) {
-		return(do_add_msg(argc,argv));
+		return(add_msg_wrapper(argc,argv));
 	}
 
 	for (i = 0; i < MsgDupElist->nelements; i++) {
@@ -1355,7 +1610,23 @@ int do_dump_add_msg(argc,argv)
 			break;
 		}
 	}
-	return(do_add_msg(argc,argv));
+	return(add_msg_wrapper(argc,argv));
+}
+
+
+/* This is identical to the usual call_element function, except
+** this just ignores it if the element is not there */
+#ifdef __STDC__
+char* do_dump_call(int argc, char** argv) {
+#else
+char* do_dump_call(argc, argv)
+int argc;
+char** argv;
+{
+#endif
+	if (GetElement(argv[1]))
+		return do_call_element(argc,argv);
+	return NULL;
 }
 
 #undef DUMPLINEWIDTH
