@@ -1,7 +1,13 @@
-static char rcsid[] = "$Id: sim_event.c,v 1.3 1997/07/18 02:58:25 dhb Exp $";
+static char rcsid[] = "$Id: sim_event.c,v 1.6 2001/04/25 17:17:01 mhucka Exp $";
 
 /*
 ** $Log: sim_event.c,v $
+** Revision 1.6  2001/04/25 17:17:01  mhucka
+** Misc. small changes to improve portability and address compiler warnings.
+**
+** Revision 1.4  2000/06/12 04:52:44  mhucka
+** Added alternate printf statement for when FLOATTIME is defined.
+**
 ** Revision 1.3  1997/07/18 02:58:25  dhb
 ** Fix for getopt problem; getopt(), optopt and optind are now
 ** G_getopt(), G_optopt and G_optind.
@@ -16,6 +22,8 @@ static char rcsid[] = "$Id: sim_event.c,v 1.3 1997/07/18 02:58:25 dhb Exp $";
  *
 */
 
+#include <stdio.h>
+#include "shell_func_ext.h"
 #include "sim_ext.h"
 
 int	DEBUG_ExpandBuffer = 0;
@@ -73,7 +81,7 @@ Buffer 		*buffer;
 Event 		*src;
 Event 		*dst;
 {
-    bcopy(src,dst, buffer->event_size);
+    BCOPY(src,dst, buffer->event_size);
 }
 
 /*
@@ -117,7 +125,7 @@ int		event_size;
 		/*
 		** copy one event
 		*/
-		bcopy( &(old_buffer[old_buffer_ptr]),
+		BCOPY( &(old_buffer[old_buffer_ptr]),
 		&(new_buffer[new_buffer_ptr]),
 		buffer->event_size);
 		/*
@@ -129,7 +137,7 @@ int		event_size;
 	    /*
 	    ** copy the buffer->end event
 	    */
-	    bcopy( &(old_buffer[old_buffer_ptr]),
+	    BCOPY( &(old_buffer[old_buffer_ptr]),
 	    &(new_buffer[new_buffer_ptr]),
 	    buffer->event_size);
 	    /*
@@ -499,7 +507,7 @@ int 		from_index;
 	    /*
 	    ** shift the event
 	    */
-	    bcopy(&(bufptr[from_index]),&(bufptr[to_index]),
+	    BCOPY(&(bufptr[from_index]),&(bufptr[to_index]),
 	    buffer->event_size);
 	    to_index = from_index;
 	}
@@ -761,8 +769,13 @@ Event		*curptr;
 Event		*startptr;
 
     if(Debug(DEBUG_GetEventList) > 1){
+#ifdef FLOATTIME
+	    printf("GetEventList '%s' %f %f\n",
+	    Pathname(buffer),start_time,duration);
+#else
 	    printf("GetEventList '%s' %d %d\n",
 	    Pathname(buffer),start_time,duration);
+#endif
     }
 
     list_cnt = 0;
