@@ -1,4 +1,4 @@
-static char rcsid[] = "$Id: sim_extfields.c,v 1.14 1997/07/18 02:58:25 dhb Exp $";
+static char rcsid[] = "$Id: sim_extfields.c,v 1.16 2001/04/25 17:17:01 mhucka Exp $";
 
 /*
 ** Code to handle extended element fields.  GENESIS command functions are:
@@ -38,6 +38,12 @@ static char rcsid[] = "$Id: sim_extfields.c,v 1.14 1997/07/18 02:58:25 dhb Exp $
 **			  (extended objects)
 **
 ** $Log: sim_extfields.c,v $
+** Revision 1.16  2001/04/25 17:17:01  mhucka
+** Misc. small changes to improve portability and address compiler warnings.
+**
+** Revision 1.15  2001/04/01 19:31:21  mhucka
+** Small cleanup fixes involving variable inits.
+**
 ** Revision 1.14  1997/07/18 02:58:25  dhb
 ** Fix for getopt problem; getopt(), optopt and optind are now
 ** G_getopt(), G_optopt and G_optind.
@@ -106,6 +112,8 @@ static char rcsid[] = "$Id: sim_extfields.c,v 1.14 1997/07/18 02:58:25 dhb Exp $
  *
 */
 
+#include <stdio.h>
+#include "shell_func_ext.h"
 #include "sim_ext.h"
 
 int AddExtField(element)
@@ -715,11 +723,11 @@ char**	argv;
 
 	GenesisObject*	object;
 	Element*	element;
-	Element*	indElement;
+	Element*	indElement = NULL;
 	char*		fieldName;
 	char*		fieldDesc;
 	char*		indElmName;
-	char*		indFieldName;
+	char*		indFieldName = NULL;
 	int		status;
 
 	fieldDesc = NULL;
