@@ -1,7 +1,10 @@
-static char rcsid[] = "$Id: sim_name.c,v 1.3 1994/10/18 15:40:19 dhb Exp $";
+static char rcsid[] = "$Id: sim_name.c,v 1.4 2001/04/25 17:17:02 mhucka Exp $";
 
 /*
 ** $Log: sim_name.c,v $
+** Revision 1.4  2001/04/25 17:17:02  mhucka
+** Misc. small changes to improve portability and address compiler warnings.
+**
 ** Revision 1.3  1994/10/18 15:40:19  dhb
 ** Name() now frees previous element name.
 **
@@ -13,8 +16,11 @@ static char rcsid[] = "$Id: sim_name.c,v 1.3 1994/10/18 15:40:19 dhb Exp $";
  *
 */
 
+#include <stdio.h>
+#include "shell_func_ext.h"
 #include "sim_ext.h"
 #include "hash.h"
+
 static HASH *name_hash_table;
 
 Name(element,name)
