@@ -1,7 +1,13 @@
-static char rcsid[] = "$Id: sim_object.c,v 1.11 1997/08/08 20:14:04 dhb Exp $";
+static char rcsid[] = "$Id: sim_object.c,v 1.14 2001/06/29 21:28:55 mhucka Exp $";
 
 /*
 ** $Log: sim_object.c,v $
+** Revision 1.14  2001/06/29 21:28:55  mhucka
+** Fixes from Hugo C.
+**
+** Revision 1.13  2001/04/25 17:17:02  mhucka
+** Misc. small changes to improve portability and address compiler warnings.
+**
 ** Revision 1.11  1997/08/08 20:14:04  dhb
 ** Fixed return value in error case and return success value in
 ** normal case for so_setobjenv().
@@ -47,6 +53,8 @@ static char rcsid[] = "$Id: sim_object.c,v 1.11 1997/08/08 20:14:04 dhb Exp $";
  *
 */
 
+#include <stdio.h>
+#include "shell_func_ext.h"
 #include "sim_ext.h"
 #include "hash.h"
 
@@ -126,7 +134,7 @@ GenesisObject*	cp_object;
 	Error(); printf("out of memory\n");
 	return 0;
       }
-    bcopy(object,cp_object,sizeof(GenesisObject));
+    BCOPY(object,cp_object,sizeof(GenesisObject));
 
     return ObjectHashEnter(cp_object);
 }
@@ -237,6 +245,7 @@ PFI	function;
 int	envarg;
 char	*ptr;
 char	*FieldHashFind();
+char	*FieldHashFindAndCopy();
 
     if(argc < 5){
 	printf("usage: %s name data function class\n", argv[0]);
@@ -250,7 +259,7 @@ char	*FieldHashFind();
 	return 0;
     }
     function_name = argv[3];
-    bzero(&object,sizeof(GenesisObject));
+    BZERO(&object,sizeof(GenesisObject));
     /*
     ** make an empty object
     */
@@ -276,25 +285,30 @@ char	*FieldHashFind();
     ** Add all fields to fieldlist setting default protection
     */
 
-    ptr = FieldHashFind(object.type);
+    ptr = FieldHashFindAndCopy(object.type);
     if (ptr != NULL)
       {
-	char	string[1000];
+	/* char	string[1000]; */
 	char*	fieldname;
 	char*	strtok();
 
-	strcpy(string,ptr);
-	fieldname = strtok(string, " \t\n");
+	/* strcpy(string,ptr); */
+	fieldname = strtok(/* string */ptr, " \t\n");
 	while (fieldname != NULL)
 	  {
 	    if (!AddFieldList(&object, fieldname))
 	      {
 		printf("do_add_object: error adding to field list\n");
+
+		free(ptr);
+
 		return 0;
 	      }
 
 	    fieldname = strtok(NULL, " \t\n");
 	  }
+
+	free(ptr);
       }
 
     nxtarg = 3;
@@ -505,9 +519,9 @@ void BasicObjects()
 {
 GenesisObject	object;
 Element*	defaults;
-char*		ptr;
+/* char*		ptr; */
 
-    bzero(&object,sizeof(GenesisObject));
+    BZERO(&object,sizeof(GenesisObject));
     /*
     ** make an empty object
     */
@@ -542,7 +556,7 @@ extern int Strcmp();
     ** count the objects
     */
     for(i=0;i<object_hash_table->size;i++){
-	if(name = object_hash_table->entry[i].key){
+	if ((name = object_hash_table->entry[i].key)) {
 	    cnt++;
 	}
     }
@@ -553,7 +567,7 @@ extern int Strcmp();
     namelist = (char **)malloc(sizeof(char *)*cnt);
     cnt = 0;
     for(i=0;i<object_hash_table->size;i++){
-	if(name = object_hash_table->entry[i].key){
+	if ((name = object_hash_table->entry[i].key)) {
 	    namelist[cnt++] = name;
 	}
     }
@@ -584,7 +598,7 @@ GenesisObject	*object;
 	printoptusage(argc, argv);
 	return;
       }
-    if(object = GetObject(optargv[1])){
+    if ((object = GetObject(optargv[1]))) {
 	ShowObject(object);
     } else {
 	Error();
