@@ -1,7 +1,45 @@
-static char rcsid[] = "$Id: eval.c,v 1.29 1997/07/18 21:46:11 dhb Exp $";
+static char rcsid[] = "$Id: eval.c,v 1.40 2001/04/25 17:17:04 mhucka Exp $";
 
 /*
 ** $Log: eval.c,v $
+** Revision 1.40  2001/04/25 17:17:04  mhucka
+** Misc. small changes to improve portability and address compiler warnings.
+**
+** Revision 1.38  2000/10/09 23:01:40  mhucka
+** Portability fixes.
+**
+** Revision 1.37  2000/09/21 19:42:37  mhucka
+** 1) Wrapped definition of TRUE/FALSE in #ifndef/endif.
+** 2) Added function declarations where appropriate.
+** 3) Added {} in some if/then clauses to get gcc to shut up.
+**
+** Revision 1.36  2000/09/07 05:22:20  mhucka
+** Modified behavior so that shell commands are not interpreted unless they
+** are the first thing on the command line.  This means that things like
+** spelling errors like typing "rm" instead of "Rm" in a command such as
+** 'eval {rm * ca}' will not cause shell commands to be executed unexpectedly.
+**
+** Revision 1.35  2000/07/12 06:13:44  mhucka
+** Added #include of malloc.h.
+**
+** Revision 1.34  2000/06/12 04:55:04  mhucka
+** Added a number of typecasts to make the compilers happy.
+**
+** Revision 1.33  2000/01/28 19:36:30  mhucka
+** Fix from MCV: increase buffer size in do_cmd_args from 1000 to 10000.
+**
+** Revision 1.32  1999/12/02 09:43:29  mhucka
+** Added improvement to error message printing by M. Vanier
+**
+** Revision 1.31  1998/08/28 15:36:28  dhb
+** Fixed return value for do_getglobal() under usage error
+** condition.
+**
+** Revision 1.30  1998/04/08 16:12:32  dhb
+** Added commands addglobal, setglobal and getglobal so that scripts
+** may manipulate global script variables within script functions and
+** in server EXEC calls.
+**
 ** Revision 1.29  1997/07/18 21:46:11  dhb
 ** Fixed lax checking of returned arglist values from commands; now
 ** check for NULL arglists and yyerror() in this case.
@@ -150,26 +188,38 @@ static char rcsid[] = "$Id: eval.c,v 1.29 1997/07/18 21:46:11 dhb Exp $";
 */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <errno.h>
 #include <setjmp.h>
+#include <malloc.h>
 #include <math.h>
+#include <string.h>
 #include "parse.h"
 #include "y.tab.h"
 #include "symtab.h"
 /* mds3 changes */
+#include "shell_func_ext.h"
+#include "ss_func_ext.h"
 #include "system_deps.h"
 
 extern char*	G_optopt;
 extern int	optargc;
 extern char**	optargv;
+extern int      yyerror();
 
-#define	FALSE	0
-#define	TRUE	(!FALSE)
+#ifndef TRUE
+#define TRUE	1
+#define FALSE	0
+#endif
 
+static char float_format[10];
 
 extern char *strsave();
 extern char *CopyString();
-static char float_format[10];
+extern int   initopt();
+extern int   G_getopt();
+extern int   printoptusage();	
+extern SIGTYPE sig_msg_restore_context();
 
 typedef struct _script_info
   {
@@ -186,13 +236,9 @@ typedef struct _script_info
 **	in the program section of the yacc source file.
 */
 
-
 extern int errno;
 extern char* TokenStr();
 extern Symtab GlobalSymbols;
-extern char *CurrentScriptName();
-
-extern void PTCall(); /* declare return type pf PTCall() */
 
 typedef struct _localvars
   {
@@ -220,13 +266,18 @@ Result		ReturnResult;
 static int	InLoop = 0;
 static ScriptInfo ScriptLoc = { 0, "<stdin>", 1 };
 
+/*
+** Forward declarations
+*/
+char* get_glob_val();
 
 
 ScriptInfo *MakeScriptInfo()
 
 {	/* MakeScriptInfo --- Return current script anem and line number */
 
-	ScriptInfo	*si;
+	ScriptInfo *si;
+	extern int yerror();
 
 	si = (ScriptInfo *) malloc(sizeof(ScriptInfo));
 	if (si == NULL)
@@ -580,7 +631,7 @@ PopLocalVars()
 
 {	/* PopLocalVars --- Deallocate local variable storage */
 
-	LocalVars	*localvars;
+	LocalVars  *localvars;
 
 	if (CurLocals == NULL) {
 		if (IsSilent() < 0)
@@ -714,17 +765,22 @@ TraceScript()
 	lv = CurLocals;
 	while (lv != NULL)
 	  {
-	    if (lv->lv_si)
+	    if (lv->lv_si) {
 		if (lv->lv_function != NULL)
 		    printf("function %s in <%s> line %d", lv->lv_function,
 				lv->lv_si->si_script, lv->lv_si->si_lineno);
 		else
 		    printf("<%s> line %d", lv->lv_si->si_script,
 						    lv->lv_si->si_lineno);
+	    }
 
 	    lv = lv->lv_prev;
 	    if (lv != NULL)
-		printf(" <- ");
+              {
+		/* printf(" <- "); */
+                /* MCV change; this looks better: */
+		printf("\n... ");
+              }
 	  }
 	printf("\n");
 
@@ -1311,9 +1367,6 @@ Result	*arg2;
 }	/* promote */
 
 
-
-extern Result ExecuteCommand();
-
 static Result do_cmd();
 
 char *do_cmd_args(arg, argc, argv)
@@ -1327,7 +1380,7 @@ char		*argv[];
 	Result	r;
 	char	**cargv;
 	char	*argcp, *arg1, *arg2, *result;
-	char    buf[1000];
+	char    buf[10000];
 
 	/*
 	** Format argument list for command execution.
@@ -1340,7 +1393,7 @@ char		*argv[];
 	  {
 
 	  case COMMAND:
-	    r = do_cmd(arg->pn_val.r.r_str, arg->pn_left);
+	    r = do_cmd(arg->pn_val.r.r_str, arg->pn_left, FALSE);
 	    switch (r.r_type)
 	      {
 
@@ -1350,7 +1403,7 @@ char		*argv[];
 
 	      case FLOAT:
 		/* sprintf(buf, "%0.20g", r.r.r_float); */
-	    sprintf(buf, float_format, r.r.r_float);
+		sprintf(buf, float_format, r.r.r_float);
 		return(strsave(buf));
 
 	      case STR:
@@ -1470,11 +1523,10 @@ char		*argv[];
 
 
 
-static Result do_cmd(cmdname, args)
-
+static Result do_cmd(cmdname, args, do_autoshell)
 char*		cmdname;
 ParseNode	*args;
-
+short		do_autoshell;
 {	/* do_cmd --- Call command and return result */
 
 	int	i;
@@ -1482,25 +1534,29 @@ ParseNode	*args;
 	int	saveinloop;
 	char	*argv[1000];
 	char	argbuf[1000];
+	void	TurnOnAutoshell(), TurnOffAutoshell();
 	jmp_buf	save;
 	Result	r;
+	extern Result ExecuteCommand();
 
 	sprintf(argbuf, "%s", cmdname);
 	argc = 1;
 	argv[0] = argbuf;
+	SetAutoshell(FALSE);
 	do_cmd_args(args, &argc, argv);
 	argv[argc] = NULL;
 
-	bcopy(ReturnBuf, save, sizeof(jmp_buf));
+	BCOPY(ReturnBuf, save, sizeof(jmp_buf));
 	saveinloop = InLoop;
 	if (setjmp(ReturnBuf) == 0)
 	  {
+	    SetAutoshell(do_autoshell);
 	    r = ExecuteCommand(argc, argv);
 	  }
 	else
 	    r = ReturnResult;
 	InLoop = saveinloop;
-	bcopy(save, ReturnBuf, sizeof(jmp_buf));
+	BCOPY(save, ReturnBuf, sizeof(jmp_buf));
 
 	for (i = 1; i < argc; i++)
 	    free(argv[i]);
@@ -1532,14 +1588,14 @@ ParseNode	*args;
 	do_cmd_args(args, &argc, argv);
 	argv[argc] = NULL;
 
-	bcopy(ReturnBuf, save, sizeof(jmp_buf));
+	BCOPY(ReturnBuf, save, sizeof(jmp_buf));
 	saveinloop = InLoop;
 	if (setjmp(ReturnBuf) == 0)
 	  {
-	    PushLocalVars(argc, argv, NULL);
+	    PushLocalVars(argc, argv, (Symtab *)NULL);
 	  }
 	InLoop = saveinloop;
-	bcopy(save, ReturnBuf, sizeof(jmp_buf));
+	BCOPY(save, ReturnBuf, sizeof(jmp_buf));
 
 	for (i = 1; i < argc; i++)
 	    free(argv[i]);
@@ -1568,6 +1624,7 @@ char	*argv[];
 	Result		*rp;
 	Result		r;
 	ParseNode	*func;
+	extern Result ExecuteCommand();
 
 	if (argc < 1)
 	    yyerror("ExecuteFunction: argc < 1!");
@@ -1579,9 +1636,9 @@ char	*argv[];
 	else
 	  {
 	    func = (ParseNode *) rp->r.r_str;
-	    PushLocalVars(argc, argv, func->pn_val.r.r_str);
+	    PushLocalVars(argc, argv, (Symtab *)func->pn_val.r.r_str);
 
-	    bcopy(ReturnBuf, save, sizeof(jmp_buf));
+	    BCOPY(ReturnBuf, save, sizeof(jmp_buf));
 	    saveinloop = InLoop;
 	    InLoop++;
 	    if (setjmp(ReturnBuf) == 0)
@@ -1594,7 +1651,7 @@ char	*argv[];
 	    else
 		r = ReturnResult;
 	    InLoop = saveinloop;
-	    bcopy(save, ReturnBuf, sizeof(jmp_buf));
+	    BCOPY(save, ReturnBuf, sizeof(jmp_buf));
 
 	    PopLocalVars();
 	  }
@@ -1635,6 +1692,7 @@ char	*argv[];
 	Result		*rp;
 	Result		r;
 	ParseNode	*func;
+	extern Result ExecuteCommand();
 
 	if (argc < 1)
 	    yyerror("ExecuteStrFunction: argc < 1!");
@@ -1648,9 +1706,9 @@ char	*argv[];
 	else
 	  {
 	    func = (ParseNode *) rp->r.r_str;
-	    PushLocalVars(argc, argv, func->pn_val.r.r_str);
+	    PushLocalVars(argc, argv, (Symtab *) func->pn_val.r.r_str);
 
-	    bcopy(ReturnBuf, save, sizeof(jmp_buf));
+	    BCOPY(ReturnBuf, save, sizeof(jmp_buf));
 	    saveinloop = InLoop;
 	    InLoop++;
 	    if (setjmp(ReturnBuf) == 0)
@@ -1663,7 +1721,7 @@ char	*argv[];
 	    else
 		r = ReturnResult;
 	    InLoop = saveinloop;
-	    bcopy(save, ReturnBuf, sizeof(jmp_buf));
+	    BCOPY(save, ReturnBuf, sizeof(jmp_buf));
 
 	    PopLocalVars();
 	  }
@@ -1709,10 +1767,10 @@ ParseNode	*args;
 	argv[0] = argbuf;
 	argc = 1;
 	do_cmd_args(args, &argc, argv);
-	PushLocalVars(argc, argv, func->pn_val.r.r_str);
+	PushLocalVars(argc, argv, (Symtab *) func->pn_val.r.r_str);
 
-	bcopy(ReturnBuf, save, sizeof(jmp_buf));
-	bcopy(BreakBuf, savebrk, sizeof(jmp_buf));
+	BCOPY(ReturnBuf, save, sizeof(jmp_buf));
+	BCOPY(BreakBuf, savebrk, sizeof(jmp_buf));
 	saveinloop = InLoop;
 	InLoop++;
 	if (setjmp(ReturnBuf) == 0)
@@ -1725,8 +1783,8 @@ ParseNode	*args;
 	else
 	    r = ReturnResult;
 	InLoop = saveinloop;
-	bcopy(save, ReturnBuf, sizeof(jmp_buf));
-	bcopy(savebrk, BreakBuf, sizeof(jmp_buf));
+	BCOPY(save, ReturnBuf, sizeof(jmp_buf));
+	BCOPY(savebrk, BreakBuf, sizeof(jmp_buf));
 
 	for (i = 1; i < argc; i++)
 	    free(argv[i]);
@@ -1752,7 +1810,7 @@ ParseNode*	argcomplist;
 	r.r_type = INT;
 	r.r.r_int = 0;
 
-	cmdtrailer = do_cmd_args(argcomplist, NULL, NULL);
+	cmdtrailer = do_cmd_args(argcomplist, (int *)NULL, (char **)NULL);
 	if (cmdtrailer == NULL)
 	    fullcmd = cmdleader;
 	else
@@ -1770,9 +1828,9 @@ ParseNode*	argcomplist;
 
 	rp = SymtabLook(&GlobalSymbols, fullcmd);
 	if (rp == NULL || rp->r_type != FUNCTION)
-	    r = do_cmd(fullcmd, args);
+	    r = do_cmd(fullcmd, args, FALSE);
 	else
-	    r = do_func(rp->r.r_str, args);
+	    r = do_func((ParseNode *) rp->r.r_str, args);
 
 	if (fullcmd != cmdleader)
 	    free(fullcmd);
@@ -2107,7 +2165,7 @@ ParseNode	*pn;
 
 	  case FUNCTION:
 	    if (pn->pn_right == NULL)
-		r = do_func(pn->pn_val.r.r_str, pn->pn_left);
+		r = do_func((ParseNode *) pn->pn_val.r.r_str, pn->pn_left);
 	    else
 	      {
 		r = do_funcwithnode(pn->pn_val.r.r_str, pn->pn_left, pn->pn_right);
@@ -2138,7 +2196,7 @@ ParseNode	*pn;
 	    /* No Return */
 
 	  case COMMAND:
-	    r = do_cmd(pn->pn_val.r.r_str, pn->pn_left);
+	    r = do_cmd(pn->pn_val.r.r_str, pn->pn_left, TRUE);
 	    break;
 
 	  case INCLUDE:
@@ -2153,12 +2211,12 @@ ParseNode	*pn;
 
 	  case SL:
 	    PTCall(pn->pn_left);
-	    SetLine(pn->pn_val.r.r_str);
+	    SetLine((ScriptInfo *) pn->pn_val.r.r_str);
 	    PTCall(pn->pn_right);
 	    break;
 
 	  case IF:
-	    SetLine(pn->pn_val.r.r_str);
+	    SetLine((ScriptInfo *) pn->pn_val.r.r_str);
 	    r = PTEval(pn->pn_left);
 	    CastToInt(&r);
 	    if (r.r.r_int)
@@ -2173,31 +2231,31 @@ ParseNode	*pn;
 	    if (rp->r_type == LOCREF)
 		rp = locref(rp->r.r_loc.l_offs);
 
-	    bcopy(BreakBuf, save, sizeof(jmp_buf));
+	    BCOPY(BreakBuf, save, sizeof(jmp_buf));
 	    InLoop++;
 	    if (setjmp(BreakBuf) == 0)
 		do_foreach(pn->pn_left, pn->pn_right, rp);
 	    InLoop--;
-	    bcopy(save, BreakBuf, sizeof(jmp_buf));
+	    BCOPY(save, BreakBuf, sizeof(jmp_buf));
 	    break;
 
 	  case WHILE:
 	  case FOR:
-	    SetLine(pn->pn_val.r.r_str);
+	    SetLine((ScriptInfo *) pn->pn_val.r.r_str);
 	    r = PTEval(pn->pn_left);
 	    CastToInt(&r);
-	    bcopy(BreakBuf, save, sizeof(jmp_buf));
+	    BCOPY(BreakBuf, save, sizeof(jmp_buf));
 	    InLoop++;
 	    if (setjmp(BreakBuf) == 0)
 		while (r.r.r_int)
 		  {
 		    PTCall(pn->pn_right);
-		    SetLine(pn->pn_val.r.r_str);
+		    SetLine((ScriptInfo *) pn->pn_val.r.r_str);
 		    r = PTEval(pn->pn_left);
 		    CastToInt(&r);
 		  }
 	    InLoop--;
-	    bcopy(save, BreakBuf, sizeof(jmp_buf));
+	    BCOPY(save, BreakBuf, sizeof(jmp_buf));
 	    r.r_type = 0;
 	    break;
 
@@ -2309,8 +2367,6 @@ char*	name;
 
 {	/* GetScriptDouble --- Return value of a script variable as a double */
 
-	extern float Atof();
-
 	Result*	rp;
 
 	rp = SymtabLook(&GlobalSymbols,name);
@@ -2453,8 +2509,6 @@ char*	name;
 
 {	/* GetScriptStr --- Return value of a script variable as a string */
 
-	extern char* get_glob_val();
-
 	return get_glob_val(name);
 
 }	/* GetScriptStr */
@@ -2468,8 +2522,6 @@ char*	value;
 
 {	/* SetScriptStr --- Set a script variable to the given value */
 
-	extern float	Atof();
-
 	Result*	rp;
 
 	rp = SymtabLook(&GlobalSymbols,name);
@@ -2493,7 +2545,7 @@ char*	value;
 	      }
 	  }
 	else
-	    fprintf(stderr, "SetScriptInt: script variable '%s' not found\n",
+	    fprintf(stderr, "SetScriptStr: script variable '%s' not found\n",
 									name);
 }	/* SetScriptStr */
 
@@ -2628,6 +2680,118 @@ char *get_glob_val(name)
 }
 
 
+/*
+** GENESIS command interface to global script variables
+*/
+
+int do_addglobal(argc, argv)
+
+int	argc;
+char*	argv[];
+
+{
+	char*	type;
+	char*	name;
+	char*	value;
+
+	initopt(argc, argv, "type name [value]");
+	if (G_getopt(argc, argv) != 0 || optargc < 3 || 4 < optargc)
+	  {
+	    printoptusage(argc, argv);
+	    printf("	type must be one of int, float or str.\n");
+	    return 0;
+	  }
+
+	type = optargv[1];
+	name = optargv[2];
+	if (optargc == 4)
+	    value = optargv[3];
+	else
+	    value = NULL;
+
+	if (strcmp(type, "str") == 0)
+	    CreateScriptString(name);
+	else if (strcmp(type, "float") == 0)
+	    CreateScriptFloat(name);
+	else if (strcmp(type, "int") == 0)
+	    CreateScriptInt(name);
+	else
+	  {
+	    printoptusage(argc, argv);
+	    printf("	type must be one of int, float or str.\n");
+	    return 0;
+	  }
+
+	if (value != NULL)
+	    SetScriptStr(name, value);
+
+	return 1;
+}
+
+
+int do_setglobal(argc, argv)
+
+int	argc;
+char*	argv[];
+
+{
+	char*	name;
+	char*	value;
+
+	initopt(argc, argv, "name value");
+
+	name = optargv[1];
+	value = optargv[2];
+	if (G_getopt(argc, argv) != 0)
+	  {
+	    printoptusage(argc, argv);
+	    return;
+	  }
+
+	if (GetScriptStr(name) == NULL)
+	  {
+	    fprintf(stderr, "%s: script variable '%s' not found\n",
+		argv[0], name);
+
+	    return 0;
+	  }
+
+	SetScriptStr(name, value);
+	return 1;
+}
+
+
+char* do_getglobal(argc, argv)
+
+int	argc;
+char*	argv[];
+
+{
+	char*	name;
+	char*	value;
+	extern  int printoptusage();
+
+	initopt(argc, argv, "name");
+
+	name = optargv[1];
+	if (G_getopt(argc, argv) != 0)
+	  {
+	    printoptusage(argc, argv);
+	    return NULL;
+	  }
+
+	value = GetScriptStr(name);
+	if (value == NULL)
+	  {
+	    fprintf(stderr, "%s: Script variable '%s' not found\n",
+		argv[0], name);
+	    value = "";
+	  }
+
+	return CopyString(value);
+}
+
+
 void CompileScriptVars(fs, leader)
 
 FILE*	fs;
@@ -2655,7 +2819,7 @@ char*	leader;
 	      case FLOAT:
 		    fprintf(fs, "%sCreateScriptFloat(\"%s\");\n", leader,
 								se->sym_ident);
-		    fprintf(fs, "%sSetScriptDouble(\"%s\", %d);\n", leader,
+		    fprintf(fs, "%sSetScriptDouble(\"%s\", %f);\n", leader,
 					se->sym_ident, se->sym_val.r.r_float);
 		break;
 
