@@ -1,7 +1,16 @@
-static char rcsid[] = "$Id: parse.c,v 1.2 1996/05/23 23:16:59 dhb Exp $";
+static char rcsid[] = "$Id: parse.c,v 1.6 2001/04/25 17:17:04 mhucka Exp $";
 
 /*
 ** $Log: parse.c,v $
+** Revision 1.6  2001/04/25 17:17:04  mhucka
+** Misc. small changes to improve portability and address compiler warnings.
+**
+** Revision 1.4  2000/09/21 19:41:10  mhucka
+** Added function declarations where appropriate.
+**
+** Revision 1.3  2000/07/12 06:14:04  mhucka
+** Added #include of malloc.h.
+**
 ** Revision 1.2  1996/05/23 23:16:59  dhb
 ** t3d/e port
 **
@@ -12,7 +21,10 @@ static char rcsid[] = "$Id: parse.c,v 1.2 1996/05/23 23:16:59 dhb Exp $";
 
 #include <stdio.h>
 #include <errno.h>
+#include <malloc.h>
+#include <stdlib.h>
 #include "parse.h"
+#include "system_deps.h"
 
 /*
 ** parse.c
@@ -31,6 +43,7 @@ ParseNode	*right;
 
 {	/* PTNew --- Make a new parse tree node and initialze fields */
 
+        extern SIGTYPE sig_msg_restore_context();
 	ParseNode	*pn;
 
 	pn = (ParseNode *) malloc(sizeof(ParseNode));
@@ -57,6 +70,8 @@ ParseNode	*pn;
 
 {	/* PTFree --- Free an entire parse tree given root */
 
+        int PTFree();
+
 	if (pn != NULL)
 	  {
 	    PTFree(pn->pn_left);
