@@ -1,11 +1,17 @@
 /*
-** $Id: parse.h,v 1.1 1992/12/11 19:05:08 dhb Exp $
+** $Id: parse.h,v 1.2 2001/04/25 17:17:04 mhucka Exp $
 ** $Log: parse.h,v $
+** Revision 1.2  2001/04/25 17:17:04  mhucka
+** Misc. small changes to improve portability and address compiler warnings.
+**
 ** Revision 1.1  1992/12/11 19:05:08  dhb
 ** Initial revision
 **
 */
 
+#ifndef PARSE_H
+#define PARSE_H
+
 /*
 ** Parsing related type and constant definitions
 */
@@ -30,3 +36,6 @@ typedef union _yyvals
 
 extern ParseNode *PTNew();
 extern Result PTEval();
+
+
+#endif
