@@ -1,7 +1,10 @@
-static char rcsid[] = "$Id: malloc.c,v 1.1 1992/12/11 19:05:31 dhb Exp $";
+static char rcsid[] = "$Id: malloc.c,v 1.3 2001/04/25 17:17:04 mhucka Exp $";
 
 /*
 ** $Log: malloc.c,v $
+** Revision 1.3  2001/04/25 17:17:04  mhucka
+** Misc. small changes to improve portability and address compiler warnings.
+**
 ** Revision 1.1  1992/12/11 19:05:31  dhb
 ** Initial revision
 **
@@ -63,7 +66,7 @@ int	b;
 	newp = gc_malloc(b);
 	if (newp != NULL)
 	  {
-	    bcopy(p, newp, b);
+	    BCOPY(p, newp, b);
 	    /*gc_free(p);*/
 	  }
 	else
